import argparse
import gettext
import logging
import os

from gettext import gettext as _

import crash_handler

from config import prefix as PREFIX
from settings import Settings

class Logger(logging.getLoggerClass()):
    def notify(message):
        os.system('notify-send --icon=covergloobus CoverGloobus "' + message + '"')

logging.setLoggerClass(Logger)

# ============================================================================ #

gettext.bindtextdomain('covergloobus', PREFIX + '/share/locale/')
gettext.textdomain('covergloobus')

c = logging.getLogger()
handler = logging.StreamHandler()
formatter = logging.Formatter('%(name)s:%(levelname)s: %(message)s')
handler.setFormatter(formatter)
c.addHandler(handler)

config = Settings()

# ============================================================================ #

def init():
    '''Parses commandline arguments, sets loggin level, loads configuration'''

    parser = argparse.ArgumentParser(description='CoverGloobus.')
    parser.add_argument('--debug', dest='debug', action='store_true',
         default=False, help=_('print debug messages'))

    parser.add_argument('--default-config', dest='default_config',
        action='store_true', default=False, help=_('load default configuration'))

    args = parser.parse_args()

    if args.debug:
        c.setLevel(logging.DEBUG)
    else:
        c.setLevel(logging.INFO)

    if not args.default_config:
        config.load()
