import gtk
import pango
import webbrowser

import sys
import os
import time
import traceback
import threading

from gettext import gettext as _

from core.common import REPORT_URL

class ReportDialog(gtk.Dialog):
    def __init__(self, message, main_thread=True):
        super(ReportDialog, self).__init__(title='CoverGloobus Error Report',
            flags=gtk.DIALOG_MODAL)

        report = gtk.Button(_('Report Problem'))
        copy = gtk.Button(_('Copy'))
        close = gtk.Button(_('Close'))

        self.get_action_area().pack_start(report, False, False)
        self.get_action_area().pack_start(copy, False, False)
        self.get_action_area().pack_start(close, False, False)

        report.connect('clicked', self.report_problem)
        copy.connect('clicked', self.copy_text, message)
        close.connect('clicked', lambda b: self.response(gtk.RESPONSE_REJECT))

        label = gtk.Label()
        label.set_markup("<b>" + _("It looks like an error has occurred.") + "</b>")
        label.set_alignment(0.5, 0.5)
        self.get_content_area().pack_start(label, False, False)

        # TextView with error message
        buff = gtk.TextBuffer()
        buff.set_text(message)
        view = gtk.TextView()
        view.set_buffer(buff)
        view.set_editable(False)
        try:
            view.modify_font(pango.FontDescription("monospace 8"))
        except Exception:
            print >> sys.stderr, "covergloobus: modify_font raised an exception"

        # allow scrolling of textview
        scrolled = gtk.ScrolledWindow()
        scrolled.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
        scrolled.add_with_viewport(view)
        scrolled.set_size_request(250, 250)

        # hide the textview in an Expander widget
        expander = gtk.expander_new_with_mnemonic(_("_Details"))
        expander.add(scrolled)
        self.get_content_area().pack_start(expander, False)

        self.show_all()

    def copy_text(self, button, text):
        clipboard = gtk.clipboard_get()
        clipboard.set_text(text)
        clipboard.store()

    def report_problem(self, button):
        webbrowser.open(REPORT_URL)


def _excepthook(exctype, value, tracebk, thread=None):
    """This function will replace sys.excepthook."""
    #show_error_window(message, main_thread)
    if exctype == KeyboardInterrupt:
        _old_excepthook(exctype, value, tracebk)
    else:
        # create traceback string and print it
        message = "".join(traceback.format_exception(exctype, value, tracebk))
        main_thread = True
        if thread:
            if not isinstance(thread, threading._MainThread):
                message = "Exception in thread %s:\n%s" % (thread.getName(), tb)
                main_thread = False
        print >> sys.stderr, message

        dialog = ReportDialog(message, main_thread)
        dialog.run()
        dialog.destroy()

        if main_thread:
            raise SystemExit(1)

def crash_handler(run):
    """a decorator for the run() method of threading.Thread.
    #Example 1:
    class ExampleThread(threading.Thread):
        @crash_handler
        def run(self):
            1 / 0 # this error will be caught by crash_handler

    #Example 2:
    def function(arg):
        arg / 0 # this error will be caught by gtkcrashhandler
    threading.Thread(target=crash_handler(function), args=(1,)).start()
    """
    def wrapper(*args, **kwargs):
        try:
            run(*args, **kwargs)
        except Exception as e:
            gobject.idle_add(_excepthook,
                e.__class__, e, sys.exc_info()[2], threading.currentThread())
    return wrapper

_old_excepthook = sys.excepthook
sys.excepthook = _excepthook
