# Copyright (C) 2009-2010 Gloobus Developers
#
#This program is free software; you can redistribute it and/or
#modify it under the terms of the GNU General Public License
#as published by the Free Software Foundation; either version 2
#of the License, or (at your option) any later version.
#
#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

import cairo
import dbus
import gtk
import gobject
import logging

from gettext import gettext as _

from core import config
from core.common import ABS_PATH, ICON_PATH
from core.common import COVER, STATE, TIME, TRACK

c = logging.getLogger(__name__)

INSTALL_MESSAGE = _('Remember to pin dock icon and restart CoverGloobus if you intend to use it on dock in future')

def Dock(cg):
    names = cg.dbus_iface.ListNames()
    if 'org.freedesktop.DockManager' in names:
        path = None
        win = None
        bus = dbus.SessionBus()
        obj = bus.get_object('org.freedesktop.DockManager',
            '/org/freedesktop/DockManager')
        iface = dbus.Interface(obj, 'org.freedesktop.DockManager')
        try:
            path = iface.GetItemsByName('CoverGloobus')[0]
        except:
            pass
        if path:
            return DockItem(path, cg)
        else:
            win = DockWindow()
            gobject.timeout_add(2000,dock_get_async, cg, iface, win)
    elif 'org.gnome.Docky' in names:
        path = None
        win = None
        bus = dbus.SessionBus()
        obj = bus.get_object('org.gnome.Docky', '/org/gnome/Docky')
        iface = dbus.Interface(obj, 'org.gnome.Docky')
        try:
            paths = iface.DockItemPaths()
            for p in paths:
                o = bus.get_object('org.gnome.Docky', p)
                if o.GetName() == 'CoverGloobus':
                    path = p
                    break
        except:
            pass
        if path:
            return DockyItemOld(path, cg)
        else:
            win = DockWindow()
            gobject.timeout_add(2000,docky_old_get_async, cg, iface, win)
    return None

def dock_get_async(cg, iface, win):
    path = None

    try:
        path = iface.GetItemByXid(win.window.xid)
    except:
        pass
    if path:
        item = DockItem(path, cg)
        item.win = win
        #item.connect()
        item.init()
        cg.dock = item
        c.notify(INSTALL_MESSAGE)
    else:
        c.error("Docky: something gone wrong with docky")
    return False

def docky_old_get_async(cg, iface, win):
    path = None

    try:
        path = iface.DockItemPathForWindowXID(win.window.xid)
    except:
        pass
    if path:
        item = DockyItemOld(path, cg)
        item.win = win
        #item.connect()
        item.init()
        cg.dock = item
        cg.add_timer(item.set_time)
        c.notify(INSTALL_MESSAGE)
    else:
        c.error("Docky: something gone wrong with docky")
    return False

class DockItem(object):
    dbus_name = 'org.freedesktop.DockManager'
    dbus_iface = 'org.freedesktop.DockItem'
    dbus_path = None

    nocover = ABS_PATH + '/images/nocover.png'
    cover = "/tmp/cover.png"
    last = None
    ids = {}

    connected = False
    win = None
    cg = None

    track_id = None
    cover_id = None
    timer_id = None

    def __init__(self, path, cg):
        self.dbus_path = path
        self.cg = cg
        self._observers = []

        bus = dbus.SessionBus()
        obj = bus.get_object(self.dbus_name, path)
        self.iface = dbus.Interface(obj, self.dbus_iface)
        self.connect()

    def init(self):
        while self._observers != []:
            self.cg.detach(self._observers.pop())
        ob = self.cg.attach(TRACK, lambda t: self.set_title('%s - %s'%(t.artist, t.title)))
        self._observers.append(ob)
        ob = self.cg.attach(COVER, lambda c: self.set_cover(c))
        self._observers.append(ob)
        if config.getboolean('Docky', 'time'):
            ob = self.cg.attach(TIME, lambda t: self.set_time(t))
            self._observers.append(ob)

    def connect(self):
        if self.connected:
            return
        bus = dbus.SessionBus()

        bus.add_signal_receiver(self.menu_item_activated, "MenuItemActivated",
            self.dbus_iface, self.dbus_name, self.dbus_path)
        bus.add_signal_receiver(self.item_confirmation_needed, "ItemConfirmationNeeded",
            self.dbus_iface, self.dbus_name, self.dbus_path)

        self.playpause = self.add_menu_item(_("Play/Pause"), container=_("Controls"))
        self.next = self.add_menu_item(_("Next"), container=_("Controls"))
        self.prev = self.add_menu_item(_("Previous"), container=_("Controls"))
        self.lyrics = self.add_menu_item(_("Show Lyrics"), container=_("Song"))
        self.tabs = self.add_menu_item(_("Show Tabs"), container=_("Song"))
        self.config = self.add_menu_item(_("Settings"))
        self.about = self.add_menu_item(_("About"))

        self.connected = True

    def disconnect(self):
        if not self.connected:
            return

        self.reset()

        bus = dbus.SessionBus()

        bus.remove_signal_receiver(self.menu_item_activated, "MenuItemActivated",
            self.dbus_iface, self.dbus_name, self.dbus_path)
        bus.remove_signal_receiver(self.item_confirmation_needed, "ItemConfirmationNeeded",
            self.dbus_iface, self.dbus_name, self.dbus_path)

        if self.win:
            self.win.destroy()

        for ob in self._observers:
            self.cg.detach(ob)
        self._observers = []

        self.connected = False

    def reset(self):
        try:
            self.iface.UpdateDockItem({'icon-file': ''})
            self.iface.UpdateDockItem({'tooltip': ''})
            self.iface.UpdateDockItem({'badge': ''})
            for k,v in self.ids.iteritems():
                self.iface.RemoveMenuItem(k)
        except:
            pass

    def set_title(self, title):
        try:
            self.iface.UpdateDockItem({'tooltip': title})
        except:
            pass #TODO: check if docky is active a reconnect

    def set_time(self, time):
        try:
            self.iface.UpdateDockItem(
                { 'badge': '%02d:%02d' % (time/60, time%60) })
        except:
            pass #TODO: check if docky is active a reconnect

    def set_cover(self, cover):
        if cover == self.last:
            return
        if cover:
            try:
                self.load_image(cover)
                self.iface.UpdateDockItem({'icon-file': self.cover})
                self.last = cover
                return
            except:
                pass
        try:
            self.iface.UpdateDockItem({'icon-file': self.nocover})
            self.last = ""
        except:
            pass #TODO: check if docky is active a reconnect

    def load_image(self,path):
        pixbuf = gtk.gdk.pixbuf_new_from_file_at_size(path, 256, 256)
        pixbuf.save(self.cover, 'png')
        return

    def add_menu_item(self, name, icon_file=None, icon_name=None, container=None):
        data = {'label':name}
        if icon_file:
            data['icon-file'] = icon_file
        if icon_name:
            data['icon-file'] = icon_name
        if container:
            data['container-title'] = container

        menu_id = self.iface.AddMenuItem(data)
        self.ids[menu_id] = name
        return menu_id

    def menu_item_activated(self, args):
        if args == self.playpause:
            self.cg.player.play(None,None)
        if args == self.next:
            self.cg.player.next(None,None)
        if args == self.prev:
            self.cg.player.previous(None,None)
        if args == self.config:
            self.cg.menu_option_config(None,None)
        if args == self.lyrics:
            self.cg.menu_option_lyrics(None,None)
        if args == self.tabs:
            self.cg.menu_option_tabs(None,None)
        if args == self.about:
            self.cg.menu_option_about(None,None)

    def item_confirmation_needed(self):
        for k,v in self.ids.iteritems():
            self.iface.ConfirmMenuItem(k)


class DockyItemOld(DockItem):
    dbus_name = 'org.gnome.Docky'
    dbus_iface = 'org.gnome.Docky.Item'

    def set_title(self, title):
        self.iface.SetText(title)

    def set_cover(self, cover):
        if cover == self.last:
            return
        if cover:
            #try:
                self.load_image(cover)
                self.iface.SetIcon(self.cover)
                self.last = cover
                return
            #except:
            #    pass
        self.iface.SetIcon(self.nocover)
        self.last = ""

    def add_menu_item(self, name, icon_file=None, icon_name=None, container=''):
        if icon_file:
            icon = icon_file
        elif icon_name:
            icon = icon_name
        else:
            icon = ''

        menu_id = self.iface.AddMenuItem(name, icon, container)
        self.ids[menu_id] = name
        return menu_id

    def reset(self):
        self.iface.ResetIcon()
        self.iface.ResetText()
        for k,v in self.ids.iteritems():
            self.iface.RemoveItem(k)

    def item_confirmation_needed (self):
        for k,v in self.ids.iteritems():
            self.iface.ConfirmItem(k)


class DockWindow(gtk.Window):
    '''
        A fake invisible window only for docky
    '''
    def __init__(self):
        super(DockWindow,self).__init__()
        self.set_title("CoverGloobus")
        #self.set_icon_from_file(ICON_PATH)
        self.set_icon_name('covergloobus')
        self.set_app_paintable(True)
        self.set_colormap( self.get_screen().get_rgba_colormap())
        self.set_decorated(False)
        self.set_keep_below(True)
        self.set_size_request(1,1)
        #self.connect("destroy", gtk.main_quit)
        self.connect("expose-event", self.expose)
        self.show_all()
        self.iconify()

    def expose(self, widget, event):
        cr = self.window.cairo_create()
        cr.set_operator(cairo.OPERATOR_CLEAR)
        cr.paint()
        return False





# ============================================================================ #
# TODO: REMOVE ALL THIS AFTER TESTING:
# ============================================================================ #

#_ = gettext.gettext

DBUS_NAME = 'org.gnome.Docky'
DBUS_OBJECT = '/org/gnome/Docky'

OLD_PATH = '/org/gnome/Docky' #TODO: remove this when Docky 2.1.0 is released
NEW_PATH = '/org/freedesktop/DockManager'
new = False

FILE_PATH = "/usr/share/applications/covergloobus.desktop"

class docky:
    #launcher_dbus = None
    connected = False

    #Menu Items
    playpause = 0
    next = 0
    prev = 0
    config = 0
    lyrics = 0
    tabs = 0
    about = 0

    #Main Items
    cg = None                #The main object to launch common functions such as next prev pause stop...
    g = None

    def __init__(self,cg,g):
        self.cg = cg
        self.g = g
        self.g.debug("Creating Docky")

    def is_active(self):
        if self.DBUS_NAME in self.cg.dbus_iface.ListNames():
            return True
        else:
            return False

    def connect(self):
        self.g.debug("Docky: Connect DBUS")
        if not self.connected:
            session_bus = dbus.SessionBus()
            try:
                dbus_object = session_bus.get_object(DBUS_NAME, NEW_PATH)
                self.new = True
            except:
                dbus_object = session_bus.get_object(DBUS_NAME, OLD_PATH)
                self.new = False

            dbus_iface = dbus.Interface(dbus_object, DBUS_NAME)
            launcher_path = None
            c = 0
            print self.cg.win.window.xid
            return
            while not self.launcher_path:
                #self.launcher_path  = self.dbus_iface.DockItemPathForDesktopFile(self.FILE_PATH)
                try:
                    print 'try for older docky version'
                    self.launcher_path = dbus_iface.DockItemPathForWindowXID(self.cg.win.window.xid)
                except:
                    try:
                        print 'try for newer docky version'
                        self.launcher_path = dbus_iface.GetItemByXID(self.cg.win.window.xid)
                    except:
                        print 'failed'
                c = c+1
                if c == 10:
                    self.g.error("Docky: something gone wrong with docky :(")
                    raise SystemExit(1)
            launcher_dbus = session_bus.get_object(DBUS_NAME, launcher_path)
            #self.iface =
            self.connected = True
        self.g.debug("Docky: Connection to DBUS done")

    def disconnect(self):
        if self.connected:
            self.g.debug("Docky: disconnecting")
            session_bus = dbus.SessionBus()
            session_bus.remove_signal_receiver(self.signal_handler,
                "MenuItemActivated", 'org.gnome.Docky.Item',
                self.DBUS_NAME, self.launcher_path)
            session_bus.remove_signal_receiver(self.item_confirmation,
                "ItemConfirmationNeeded", 'org.gnome.Docky.Item',
                self.DBUS_NAME, self.launcher_path)
            self.connected = False

    # ================================= CREATE MENU ================================ #
    def create_menu(self):
        if self.connected:
            self.playpause     = self.launcher_dbus.AddMenuItem(_("Play/Pause"),"",    _("Controls"))
            self.next         = self.launcher_dbus.AddMenuItem(_("Next"),"",            _("Controls"))
            self.prev         = self.launcher_dbus.AddMenuItem(_("Previous"),"",        _("Controls"))
            self.lyrics     = self.launcher_dbus.AddMenuItem(_("Show Lyrics"),"",    _("Song"))
            self.tabs         = self.launcher_dbus.AddMenuItem(_("Show Tabs"),"",        _("Song"))
            self.config     = self.launcher_dbus.AddMenuItem(_("Settings"),"",        "")
            self.about         = self.launcher_dbus.AddMenuItem(_("About"),"",            "")
            self.g.debug("Docky: Create Menu" + str(self.playpause))

    # =============================== BIND SIGNALS ================================= #
    def bind_signals(self):
        self.g.debug("Docky: Bind Menu Signals")
        bus = dbus.SessionBus()
        bus.add_signal_receiver (self.item_confirmation,
                "ItemConfirmationNeeded", 'org.gnome.Docky.Item',
                self.DBUS_NAME, self.launcher_path)
        bus.add_signal_receiver (self.signal_handler,
                "MenuItemActivated", 'org.gnome.Docky.Item',
                self.DBUS_NAME, self.launcher_path)

    def set_title(self, artist, song):
        self.g.debug("Docky: Set Title")
        if self.connected:
            if artist == '' and song == '':
                self.launcher_dbus.SetText('CoverGloobus')
            else:
                self.launcher_dbus.SetText(artist + " - " + song)

    def set_waiting(self):
        self.g.debug("Docky: Set Waiting")
        if self.connected:
            self.launcher_dbus.SetWaiting()

    def unset_waiting(self):
        self.g.debug("Docky: Unset Waiting")
        if self.connected:
            self.launcher_dbus.UnsetWaiting()

    def set_cover(self, path):
        self.g.debug("Docky: Set Cover: " + path)
        if self.connected:
            try:
                self.round_image(path)
                self.launcher_dbus.SetIcon("/tmp/cover.png")
            except:
                self.launcher_dbus.SetIcon(ABS_PATH + "/images/nocover.png")
        else:
            self.g.warning("DOCKY: Docky Not connected")

    def round_image(self,filename):
        pixbuf = gtk.gdk.pixbuf_new_from_file(filename)
        w = pixbuf.get_width()
        h = pixbuf.get_height()

        surface = cairo.ImageSurface(0,w,h)        #Create a new cairo surface to place the image on
        #nostar    = cairo.ImageSurface.create_from_png("/home/jordi/Escriptori/star.png")

        ct = cairo.Context(surface)                #Create a context to the new surface
        self.roundedrec(ct,0,0,w,h,40)                #Rounded Rectangle
        ct.clip()                                #Clip the out of the rectangle

        ct.new_path()                            #Path not afected by clip

        ct2 = gtk.gdk.CairoContext(ct)            #Create GDK formated cairo contex
        ct2.set_source_pixbuf(pixbuf,0,0)        #Draw pixbuf to the new surface
        ct2.paint()                                #Paint

        ct2.new_path()
        ct2.set_source_rgba(1,1,1,1)
        ct2.set_line_width(10)
        self.roundedrec(ct2,0,0,w,h,40)
        ct2.stroke()

        #ct2.move_to(10,10)
        #ct2.set_source_surface(nostar)
        #ct2.scale(0.1,0.1)
        #ct2.paint()


        surface.write_to_png("/tmp/cover.png")


    def roundedrec(self,context,x,y,w,h,r = 10):
        "Draw a rounded rectangle"
        #   A****BQ
        #  H      C
        #  *      *
        #  G      D
        #   F****E

        context.move_to(x+r,y)                      # Move to A
        context.line_to(x+w-r,y)                    # Straight line to B
        context.curve_to(x+w,y,x+w,y,x+w,y+r)       # Curve to C, Control points are both at Q
        context.line_to(x+w,y+h-r)                  # Move to D
        context.curve_to(x+w,y+h,x+w,y+h,x+w-r,y+h) # Curve to E
        context.line_to(x+r,y+h)                    # Line to F
        context.curve_to(x,y+h,x,y+h,x,y+h-r)       # Curve to G
        context.line_to(x,y+r)                      # Line to H
        context.curve_to(x,y,x,y,x+r,y)             # Curve to A
        return

    #====================== DOCKY MENU SIGNAL HANDLER ================#
    def signal_handler(self,args):
        self.g.debug("Docky: an item was pressed - " + str(args))
        if args == self.playpause:
            self.cg.player.play(None,None)
        if args == self.next:
            self.cg.player.next(None,None)
        if args == self.prev:
            self.cg.player.previous(None,None)
        if args == self.config:
            self.cg.menu_option_config(None,None)
        if args == self.lyrics:
            self.cg.menu_option_lyrics(None,None)
        if args == self.tabs:
            self.cg.menu_option_tabs(None,None)
        if args == self.about:
            self.cg.menu_option_about(None,None)

    #== Docky ask for confirmations every 2 min or so, we listen to its signal and send confirmation
    def item_confirmation(self):
        self.g.debug("Docky: Menu items confirmation required")
        self.launcher_dbus.ConfirmItem(self.playpause)
        self.launcher_dbus.ConfirmItem(self.next)
        self.launcher_dbus.ConfirmItem(self.prev)
        self.launcher_dbus.ConfirmItem(self.config)
        self.launcher_dbus.ConfirmItem(self.lyrics)
        self.launcher_dbus.ConfirmItem(self.tabs)
        self.launcher_dbus.ConfirmItem(self.about)
