import logging
import os

from ConfigParser import SafeConfigParser

from gettext import gettext as _

from core.common import ABS_PATH
from core.common import CONFIG_PATH
from core.common import HOME_PATH

c = logging.getLogger(__name__)

class Settings(SafeConfigParser):
    def __init__(self):
        SafeConfigParser.__init__(self)
        self.load_default()

    def load_default(self):
        self.add_section('General')
        self.set('General', 'start_at_login', '0')
        self.set('General', 'hide_if_inactive', '0')
        self.set('General', 'show_status_icon', '1')
        self.set('General', 'player', 'Rhythmbox')

        self.add_section('Lyrics')
        self.set('Lyrics', 'lyrics_source', 'lyricswikia')
        self.set('Lyrics', 'tabs_source', 'ultimateguitar')
        self.set('Lyrics', 'widget', '0')
        self.set('Lyrics', 'window', 'new')
        self.set('Lyrics', 'refresh', '0')
        self.set('Lyrics', 'try_all', '0')
        self.set('Lyrics', 'cache', '0')
        self.set('Lyrics', 'lyricsfly_key', '1c0736f65ac693cbd-temporary.API.access')

        self.add_section('Covers')
        self.set('Covers', 'search_engines', 'Location,Amazon')
        self.set('Covers', 'search_pattern', '')
        self.set('Covers', 'location_filenames', 'folder,cover,front,album,albumart')
        self.set('Covers', 'location_extentions', 'jpg,jpeg,png,gif')

        self.add_section('Docky')
        self.set('Docky', 'show', '0')
        self.set('Docky', 'time', '1')

        self.add_section('MPD')
        self.set('MPD', 'host', 'localhost')
        self.set('MPD', 'port', '6600')
        self.set('MPD', 'password', '')
        self.set('MPD', 'path', HOME_PATH + '/Music')

        self.add_section('Clementine')
        self.set('Clementine', 'legacy', '0')

        self.add_section('Window')
        self.set('Window', 'theme', ABS_PATH + '/themes/BadChoice')
        self.set('Window', 'resize', '100')
        self.set('Window', 'stick', '0')
        self.set('Window', 'ontop', '0')
        self.set('Window', 'x', '20')
        self.set('Window', 'y', '20')
        self.set('Window', 'hide_inactive', '0')
        self.set('Window', 'show_background', '0')
        self.set('Window', 'background_opacity', '0.5')
        self.set('Window', 'background_color', '#000')
        self.set('Window', 'long_text', 'theme')

    def load(self):
        if os.path.isfile(CONFIG_PATH):
            self.read(CONFIG_PATH)
        else:
            c.warning(_('Configuration file not found'))

    def save(self):
        if not os.path.isfile(CONFIG_PATH):
            path = os.path.dirname(CONFIG_PATH)
            if not os.path.exists(path):
                c.warning(_("Creating configuration directory"))
                os.makedirs(path)

        with open(CONFIG_PATH, 'wb') as f:
            self.write(f)
            c.debug('Configuration saved')
