# Copyright (C) 2009-2010 Gloobus Developers
#
#This program is free software; you can redistribute it and/or
#modify it under the terms of the GNU General Public License
#as published by the Free Software Foundation; either version 2
#of the License, or (at your option) any later version.
#
#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

import cairo
import gobject
import gtk
import glib
import logging
import os.path
import pango
import sys

from xml.dom import minidom

from gettext import gettext as _

from core import config

from core.common import COVER, STATE, TIME, TRACK, PLAYER
from core.settings import Settings
from core.track import State

from widgets.button import Button, StateButton
from widgets.image import Image
from widgets.rating import HRating, VRating
from widgets.seekbar import HSlider, VSlider
from widgets.text import Text, Timer
from widgets import Rectangle, Reflection

c = logging.getLogger(__name__)

MIN_SIZE = 50

class CoverGloobusUI(gtk.Window):
    cg = None

    stick = False
    ontop = False

    _pt = False
    _px = 0
    _py = 0

    _widgets = []
#    _background = None
    _loaded = False
#    _track = False
#    _updated = False
    _observers = []
    width = 0
    height = 0

    def __init__(self, cg):
        super(CoverGloobusUI,self).__init__()
        self.cg = cg

        self.set_app_paintable(True)
        self.set_colormap( self.get_screen().get_rgba_colormap())
        #self.set_icon_from_file(ICON_PATH)
        self.set_icon_name('covergloobus')
        self.set_keep_below(True)
        #super(CoverGloobusUI,self).stick()
        self.set_title('CoverGloobus')
        self.set_size_request(1,1)

        self.set_type_hint(gtk.gdk.WINDOW_TYPE_HINT_DOCK)

        self.move(config.getint("Window","x"), config.getint("Window","y"))

        self.add_events(gtk.gdk.BUTTON_PRESS_MASK)
        self.add_events(gtk.gdk.BUTTON_RELEASE_MASK)
        #self.add_events(gtk.gdk.KEY_PRESS_MASK)
        self.add_events(gtk.gdk.POINTER_MOTION_MASK)

        self.set_property('accept-focus', False)

        self.connect('button-press-event', self.button_press)
        self.connect('button-release-event', self.button_release)
        self.connect('motion-notify-event', self.motion_notify)
        self.connect('expose-event', self.expose_event)

        self.show_all()

    def set_stick(self, stick):
        self.stick = stick

    def set_ontop(self, ontop):
        self.ontop = ontop
        if ontop:
            self.set_keep_above(True)
        else:
            self.set_keep_below(True)

    def button_press(self, widget, event):
        if event.button == 1 and not self.stick:
            self._pt = True
            self._px, self._py = self.get_position()
            self._px = event.x_root - self._px
            self._py = event.y_root - self._py
        return False

    def button_release(self, widget, event):
        if event.button == 1:
            self._pt = False
        return False

    def motion_notify(self, widget, event):
        if self._pt:
            self.move(int(event.x_root-self._px), int(event.y_root-self._py))
        return False

    def init(self, path=None):
        if not path:
            path = config.get("Window","theme")
        if path != self.path:
            if not os.path.exists(path + "/skin.xml"):
                c.warning(_('Theme not found'))
                c.debug('Theme not found: ' + path)
                if not self._loaded:
                    c.warning(_('Setting default theme'))
                    conf = Settings()
                    path = conf.get("Window","theme")
            c.debug("Theme path: " + path)
            self.path = path
            self.load_xml(path + "/skin.xml")
            self.scale(config.getint('Window', 'resize'))
            self.move(config.getint("Window","x"), config.getint("Window","y"))

        self.ontop = config.getboolean("Window", "ontop")
        self.stick = config.getboolean("Window", "stick")
        self.set_keep_above(self.ontop)

    def load_xml(self, filename):
        if self._loaded:
            self._unload()

        self.fixed = gtk.Fixed()
        self.add(self.fixed)

        dom = minidom.parse(filename)
        skinobj = dom.getElementsByTagName("skin")[0]
        try:
            self.width = int(skinobj.getAttribute("width"))
            self.height = int(skinobj.getAttribute("height"))
            if self.width < 1 or self.height < 1: raise Exception
            self.src_width = self.width
            self.src_height = self.height
        except:
            c.error('Error parsing theme width and height')
            sys.exit(1)
        for domitem in skinobj.childNodes:
            if domitem.nodeType == 1:
                type = domitem.nodeName
                item = False
                if type == "image":
                    self.add_image(domitem)
                elif type == "albumcover":
                    self.add_image(domitem, True)
                elif type== "rating":
                    self.add_rating(domitem)
                elif type in ('titlename', 'artistname', 'albumname'):
                    self.add_text(domitem,type[:-4])
                elif type in ('genre', 'tracknumber', 'year', 'length', 'time'):
                    self.add_text(domitem,type)
                elif type in ('quit', 'mute'):
                    self.add_button(domitem)
                elif type == "playercontrols":
                    for item in domitem.childNodes:
                        if item.nodeType == 1:
                            self.add_button(item)
                elif type == 'seekbar':
                    self.add_seekbar(domitem)
#        self._new_background()

        for w in self._widgets:
            self.fixed.put(w, w.get_x(), w.get_y())

        self._loaded = True
        self.show_all()

    def _unload(self):
        self.fixed.destroy()
        for ob in self._observers:
            self.cg.detach(ob)
        self._observers = []
#        self._images = []
        self._widgets = []
        self._loaded = False

#    def _new_background(self):
#        super(CoverGloobusUI,self).resize(self.width, self.height)
#        self._background = cairo.ImageSurface(cairo.FORMAT_ARGB32,
#            self.width, self.height)
#        self._draw_background()

#    def _draw_background(self):
#        cr = gtk.gdk.CairoContext(cairo.Context(self._background))
#        cr.set_operator(cairo.OPERATOR_CLEAR)
#        cr.paint()
#        cr.set_operator(cairo.OPERATOR_OVER)
#        if self.cg.config.getboolean('Window', 'show_background'):
#            self.draw_nice_background(cr,self.width, self.height)
#        for i in self._images: i.draw(cr)
#        self.queue_draw()

    def expose_event(self, widget, event):
#        if self._updated:
#            self._draw_background()
#            self._updated = False
        cr = self.window.cairo_create()
        cr.rectangle(event.area.x, event.area.y,
            event.area.width, event.area.height)
        cr.clip()
        cr.set_operator(cairo.OPERATOR_CLEAR)
        cr.paint()
        cr.set_operator(cairo.OPERATOR_OVER)
#        cr.set_source_surface(self._background)
#        cr.paint()
        if config.getboolean('Window', 'show_background'):
            self.draw_nice_background(cr, self.width, self.height)
        return False

    def draw_nice_background(self,cr,w,h):
        x = 0
        y = 0
        r = 15

        cr.set_operator(cairo.OPERATOR_OVER)

        alpha = config.getfloat('Window', 'background_opacity')
        try:
            color = gtk.gdk.color_parse(config.get('Window', 'background_color'))
            cr.set_source_rgba (float(color.red)/65535.0,
                                float(color.green)/65535.0,
                                float(color.blue)/65535.0,
                                alpha)
        except:
            cr.set_source_rgba (0, 0, 0, alpha)

        cr.move_to  ( x+r,y )
        cr.line_to  ( x+w-r,y )
        cr.curve_to ( x+w-r,y,x+w,y,x+w,y+r )
        cr.line_to  ( x+w,y+h-r )
        cr.curve_to ( x+w,y+h-r,x+w,y+h,x+w-r,y+h )
        cr.line_to  ( x+r,y+h )
        cr.curve_to ( x+r,y+h,x,y+h,x,y+h-r )
        cr.line_to  ( x,y+r )
        cr.curve_to ( x,y+r,x,y,x+r,y)
        cr.fill     ( )

    def scale(self, percent):
        if percent < 1:
            print 'too little percent'
            return

        self.width  = int( self.src_width  / 100.0 * percent )
        self.height = int( self.src_height / 100.0 * percent )
        if self.width < MIN_SIZE:
            self.width = MIN_SIZE
            percent = float(self.width) / (self.src_width / 100.0)
            self.height = int( self.src_height / 100.0 * percent )
        if self.height < MIN_SIZE:
            self.height = MIN_SIZE
            percent = float(self.height) / (self.src_height / 100.0)
            self.width = int( self.src_width / 100.0 * percent )
        if self.width > gtk.gdk.screen_width():
            self.width = gtk.gdk.screen_width()
            percent = float(self.width) / (self.src_width / 100.0)
            self.height = int( self.src_height / 100.0 * percent )
        if self.height > gtk.gdk.screen_height():
            self.height = gtk.gdk.screen_height()
            percent = float(self.height) / (self.src_height / 100.0)
            self.width = int( self.src_width / 100.0 * percent )

        self.resize(self.width, self.height)

#        for i in self._images:
#            i.resize(percent)

        for w in self._widgets:
            w.scale(percent)
            self.fixed.move(w, w.get_x(), w.get_y())

#        self._new_background()

    def add_button(self, domitem):
        type = domitem.nodeName

        size = self.read_size(domitem)

        data = {'normal': 'src', 'hover': 'hover', 'pressed': 'pressed'}
        for key, val in data.iteritems():
            val = domitem.getAttribute(val)
            if val:
                val = self.path + '/' + val
            data[key] = val

        if type == 'play':
            button = StateButton(size)
            button.add_state(State.PAUSED, **data)

            data = {'normal': 'pause', 'hover': 'pause_hover', 'pressed': 'pause_pressed'}
            for key, val in data.iteritems():
                val = domitem.getAttribute(val)
                if val:
                    val = self.path + '/' + val
                data[key] = val
            button.add_state(State.PLAYING, **data)

            ob = self.cg.attach(STATE, self.change_state, button)
            self._observers.append(ob)
            button.set_state(self.cg.state)
        else:
            button = Button(size, **data)

        reflect = domitem.getAttribute('reflect')
        if reflect:
            reflection = Reflection()
            if reflection.load(reflect):
                button.set_reflection(reflection)

        if type == 'prev':
            button.connect('clicked', lambda w: self.cg.player.previous())
        elif type == 'play':
            button.connect('clicked', lambda w: self.cg.player.play())
        elif type == 'next':
            button.connect('clicked', lambda w: self.cg.player.next())
        elif type == 'mute':
            button.connect('clicked', lambda w, e: self.player.mute())
        elif type == 'quit':
            button.connect('clicked', lambda w,b: self.cg.quit())
        else:
            c.error('Unsupported button type: ' + type)
            return

        self._widgets.append(button)

    def add_image(self, domitem, cover=False):
        size = self.read_size(domitem)

        if cover and not size.width or not size.height:
            c.error('Incomplete image size')
            return

        if cover:
            src = self.cg.cover
        else:
            src = domitem.getAttribute("src")
            if src:
                src  = self.path + '/' + src
            else:
                c.error(_('Image source is not defined'))
                return

        image = Image(size)

        radius = domitem.getAttribute("round")
        if radius:
            try:
                image.set_round(int(radius))
            except:
                pass

        reflect = domitem.getAttribute("reflect")
        if reflect:
            reflection = Reflection()
            if reflection.load(reflect):
                image.set_reflection(reflection)

        mask = domitem.getAttribute("mask")
        if mask:
            mask = self.path + '/' + mask
            image.set_mask(mask)

        opacity = domitem.getAttribute("opacity")
        if opacity:
            try:
                image.set_opacity(float(opacity))
            except:
                c.error(_('Wrong opacity value'))

        if cover:
            ob = self.cg.attach(COVER, lambda p: image.set_image(p))
            self._observers.append(ob)
        image.set_image(src)

        self._widgets.append(image)

    def add_rating(self, domitem):
        size = self.read_size(domitem)

        try:
            spacing = int(domitem.getAttribute("spacing"))
        except:
            spacing = 0

        src_star   = self.path + "/" + domitem.getAttribute("star")
        src_nostar = self.path + "/" + domitem.getAttribute("nostar")

        src_hover  = domitem.getAttribute("hover")
        if src_hover:
            src_hover = self.path + "/" + src_hover

        src_pressed  = domitem.getAttribute("pressed")
        if src_pressed:
            src_pressed = self.path + "/" + src_pressed

        direction = domitem.getAttribute("direction")
        if direction and direction == 'vertical':
            rating = VRating(size,spacing, src_star, src_nostar, src_hover, src_pressed)
        else:
            rating = HRating(size,spacing, src_star, src_nostar, src_hover, src_pressed)

        rating.set_clickable('rating' in self.cg.player.capabilities)

        rating.connect('set_rating', self.rating_clicked)

        reflect = domitem.getAttribute("reflect")
        if reflect:
            reflection = Reflection()
            if reflection.load(reflect):
                rating.set_reflection(reflection)

        ob = self.cg.attach(TRACK, self.change_rating, rating)
        self._observers.append(ob)
        ob = self.cg.attach(PLAYER, self.rating_clickable, rating)
        self._observers.append(ob)
        rating.set_rating(self.cg.track.rating)
        self._widgets.append(rating)

    def add_seekbar(self, domitem):
        size = self.read_size(domitem)
        background = domitem.getAttribute("back")
        if background:
            background = self.path + '/' + background
        foreground  = domitem.getAttribute("top")
        if foreground:
            foreground = self.path + '/' + foreground

        if size.width > size.height:
            seekbar = HSlider(size, background, foreground)
        else:
            seekbar = VSlider(size, background, foreground)

        for item in domitem.childNodes:
            if item.nodeType == 1 and item.nodeName == 'thumb':
                thumb_size = self.read_size(item, position=False)
                if not thumb_size:
                    break
                thumb_path = item.getAttribute('src')
                if thumb_path:
                    thumb_path = self.path + '/' + thumb_path
                else:
                    break
                top, bottom, left, right = 0, 0, 0, 0
                try:
                    t = item.getAttribute('top')
                    b = item.getAttribute('bottom')
                    l = item.getAttribute('left')
                    r = item.getAttribute('right')

                    if t:
                        top = int(t)
                    if b:
                        bottom = int(b)
                    if l:
                        left = int(l)
                    if r:
                        right = int(r)
                except:
                    pass
                seekbar.set_thumb(thumb_path, thumb_size, top, bottom, left, right)
                break


        seekbar.set_max(self.cg.track.length)
        seekbar.set_value(self.cg.time)

        ob = self.cg.attach(TIME, lambda t: seekbar.set_value(t))
        self._observers.append(ob)
        ob = self.cg.attach(TRACK, lambda t: seekbar.set_max(t.length))
        self._observers.append(ob)
        seekbar.connect('value-changed', lambda w, v: self.cg.player.set_position(v))
        self._widgets.append(seekbar)

    def add_text(self, domitem, type):
        size = self.read_size(domitem)
        #if size.width == 0:
        #    size.set_width(self.width - size.x)

        if type == 'time':
            text = Timer(size)
            ob = self.cg.attach(TIME, self.change_time, text)
            self._observers.append(ob)
        elif type == 'length':
            text = Timer(size)
            ob = self.cg.attach(TRACK, self.change_text, text, type)
            self._observers.append(ob)
        else:
            text = Text(size)
            ob = self.cg.attach(TRACK, self.change_text, text, type)
            self._observers.append(ob)

        font = domitem.getAttribute("font")
        if font:
            text.set_font(pango.FontDescription(font))

        color = domitem.getAttribute("color")
        if color:
            text.set_color(self.get_rgba(color))

        shadowcolor = domitem.getAttribute("shadowcolor")
        if shadowcolor:
            shadowcolor = self.get_rgba(shadowcolor)
            try:
                shadow_x = int(domitem.getAttribute("shadow_x"))
                shadow_y = int(domitem.getAttribute("shadow_y"))
            except:
                shadow_x = 1
                shadow_y = 1
            text.set_shadowcolor(shadowcolor, shadow_x, shadow_y)

        align = domitem.getAttribute("align")
        if align:
            text.set_align(align)

        valign = domitem.getAttribute("valign")
        if valign:
            text.set_valign(valign)

        text.set_wrap(domitem.getAttribute("wrap") == 'true')

        long_text = config.get('Window', 'long_text')
        maxchars = domitem.getAttribute("maxchars")
        ellipsize = domitem.getAttribute('ellipsize')
        if long_text == 'ellipsize-word':
            text.set_ellipsize(Text.ELLIPSIZE_WORD)
        elif long_text == 'ellipsize-char':
            text.set_ellipsize(Text.ELLIPSIZE_CHAR)
        elif long_text == 'scroll':
            text.set_scroll(True)
        elif long_text == 'theme':
            if domitem.getAttribute('scroll') == 'true':
                text.set_scroll(True)
            elif ellipsize == 'word':
                text.set_ellipsize(Text.ELLIPSIZE_WORD)
            elif ellipsize == 'char':
                text.set_ellipsize(Text.ELLIPSIZE_CHAR)
            elif maxchars:
                try:
                    text.set_maxchars(int(maxchars))
                except:
                    pass

        reflect = domitem.getAttribute('reflect')
        if reflect:
            reflection = Reflection()
            if reflection.load(reflect):
                text.set_reflection(reflection)

        if type == 'time':
            self.change_time(self.cg.time, text)
        else:
            self.change_text(self.cg.track, text, type)

        self._widgets.append(text)

    def get_rgba(self, color):
        if not color: return (1,1,1,1)
        return (
            int(color[1:3], 16) / 255.0,
            int(color[3:5], 16) / 255.0,
            int(color[5:7], 16) / 255.0,
            int(color[7:9], 16) / 255.0)

    def read_size(self, domitem, position=True, dimentions=False):
        x = domitem.getAttribute("x")
        y = domitem.getAttribute("y")
        w = domitem.getAttribute("width")
        h = domitem.getAttribute("height")
        try:
            if x:
                x = int(x)
            elif position:
                raise Exception
            else:
                x = 0
            if y:
                y = int(y)
            elif position:
                raise Exception
            else:
                y = 0

            if w:
                w = int(w)
            elif dimentions:
                raise Exception
            else:
                w = 0
            if h:
                h = int(h)
            elif dimentions:
                raise Exception
            else:
                h = 0
        except:
            c.error("Invalid size values")

        return Rectangle(x,y,w,h)

    def button_clicked(self, widget, button):
        if button == 'prev':
            self.cg.player.previous()
        elif button == 'play':
            self.cg.player.play()
        elif button == 'next':
            self.cg.player.next()
        elif button == 'mute':
            self.cg.player.mute()
        return True

    def rating_clicked(self, widget, rating):
        if 'rating' in self.cg.player.capabilities:
            c.debug('Setting rating to ' + str(rating))
            self.cg.player.set_rating(rating)
        else:
            c.debug('Rating not supported')
            widget.set_rating(0)

    def rating_clickable(self, player, rating):
        rating.set_clickable('rating' in player.capabilities)

    def change_state(self, state, button):
        if state == State.PLAYING:
            button.set_state(state)
        elif state in (State.PAUSED, State.STOPPED, State.CLOSED):
            button.set_state(State.PAUSED)

    def change_text(self, track, text, attr):
        if attr == 'length':
            text.set_time(track.length)
        elif hasattr(track, attr):
            text.set_text(str(getattr(track, attr)))

    def change_time(self, time, text):
        text.set_time(time)

    def change_rating(self, track, rating):
        rating.set_rating(track.rating)

    def show(self):
        super(CoverGloobusUI, self).show()
        super(CoverGloobusUI, self).stick()
        self.queue_draw()
