import htmlentitydefs
import os
import string
import sys
import re

from urllib2 import quote, urlopen

from core.common import ABS_PATH
from core.common import THEME_PATH

def get_themes():
    themes = []
    for path in (ABS_PATH+'/themes/' , THEME_PATH):
        for folder in os.listdir(path):
            if os.path.exists(os.path.join(path, folder, 'skin.xml')):
                themes.append(os.path.join(path, folder))
    return themes

def html_unescape(text):
    """ Removes HTML or XML character references and entities from a text string.
        from Fredrik Lundh
        http://effbot.org/zone/re-sub.htm#unescape-html

        @param text The HTML (or XML) source text.
        @return The plain text, as a Unicode string, if necessary.
    """
    def fixup(m):
        text = m.group(0)
        if text[:2] == "&#":
            # character reference
            try:
                if text[:3] == "&#x":
                    return unichr(int(text[3:-1], 16))
                else:
                    return unichr(int(text[2:-1]))
            except ValueError:
                pass
        else:
            # named entity
            try:
                text = unichr(htmlentitydefs.name2codepoint[text[1:-1]])
            except KeyError:
                pass
        return text # leave as is
    return re.sub("&#?\w+;", fixup, text)

def wikilyrics_get_album(artist, title):
    artist = quote(artist.replace(" ","_").encode('utf-8'))
    title = quote(title.replace(" ","_").encode('utf-8'))

    url = "http://lyrics.wikia.com/" + artist + ":" + title
    c.debug("Getting Album: " + url)
    try:
        webcode = urlopen(url).read()
        #        appears on the album         //First string
        #        (.*?)                        //Match the title the first time (the ? adds a nongreedy search)
        #        \(.*/>                        //What is afeter the title (its non-greedy so it will return the first one)
        regexp = re.compile ("appears on the album (.*?)\(.*/>",re.DOTALL)
        m = re.search(regexp, webcode)
        return m.group(1)
    except:
        return ''

def name_hash(s, artist=False):
    """ Prepares song title or artist name for cache file name.

        @param str: string - title or artist
        @param artist: boolean - True if str is artist name
        @return: string - prepared artist or title name
    """
    s = s.decode('utf-8').lower()
    if artist:
        s = re.sub('^the\s*', '', s)
    else:
        s = re.sub('[\[\(]\s*(live|bonus|feat|ft).*?[\]\)]', '', s)
    s = s.encode('utf-8')
    s = s.translate(string.maketrans('',''), string.punctuation + ' -')
    return s
