#!/usr/bin/env python

# Copyright (C) 2009-2010 Gloobus Developers
#
#This program is free software; you can redistribute it and/or
#modify it under the terms of the GNU General Public License
#as published by the Free Software Foundation; either version 2
#of the License, or (at your option) any later version.
#
#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

import argparse

from common import c
c.set_debug(True)

class DefaultAction(argparse.Action):
    def __call__(self, parser, namespace, values, option_string=None):
        print 'default action'

class TestLyricAction(argparse.Action):
    def __call__(self, parser, namespace, values, option_string=None):
        go = True
        if not namespace.engine:
            print 'lyrics engine name is required to test it'
            go = False
        if not namespace.artist:
            print 'artist name is required to test lyrics engine'
            go = False
        if not namespace.title:
            print 'song title is required to test lyrics engine'
            go = False

        if go:
            try:
                klass = namespace.engine
                engine = __import__('lyrics.'+klass, fromlist=['lyrics']).__dict__[klass]()
                text = engine.get(namespace.artist, namespace.title)
                if text:
                    print 'LYRICS:'
                    print text
                    print '--------------------\n'
                    print 'URL:'
                    print engine.get_webpage()
                else:
                    print 'lyrics not found'
            except Exception as e:
                print e

def no_action():
    print 'no action'

def test_lyric_engine(namespace):
    engine = None
    artist = None
    title = None

    if namespace.engine:
        engine = __import__('lyrics.'+namespace.engine,
            fromlist=['lyrics']).__dict__[namespace.engine]()
    else:
        print 'lyrics engine name is required to test it'

    if namespace.artist:
        artist = ' '.join(namespace.artist)
    else:
        print 'artist name is required to test lyrics engine'

    if namespace.title:
        title = ' '.join(namespace.title)
    else:
        print 'song title is required to test lyrics engine'

    if engine and artist and title:
        text = engine.get(artist, title)
        if text:
            print 'LYRICS:'
            print text
            print '--------------------\n'
            print 'URL:'
            print engine.get_webpage()
        else:
            print 'lyrics not found'

if __name__ == '__main__':
    parser = argparse.ArgumentParser(description='CoverGloobus testing utility.')

    parser.add_argument('--lyric', dest='action', action='store_const',
                        const=test_lyric_engine,
                        help='test a lyric engine')

    parser.add_argument('--engine', dest='engine', action='store',
                        help='engine name')

    parser.add_argument('--artist', dest='artist', action='store', nargs='+',
                        help='artist name')
    parser.add_argument('--title', dest='title', action='store', nargs='+',
                        help='song title')

    parser.add_argument('--version', action='version', version='%(prog)s 1.7')
    # TODO: load current version

    args = parser.parse_args()
    if args.action:
        try:
            args.action(args)
        except Exception as e:
            print e
    else:
        print 'no action'
