# Copyright (C) 2009-2010 Gloobus Developers
#
#This program is free software; you can redistribute it and/or
#modify it under the terms of the GNU General Public License
#as published by the Free Software Foundation; either version 2
#of the License, or (at your option) any later version.
#
#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

import logging
import os
import re
import urllib2

from tempfile import NamedTemporaryFile

from gettext import gettext as _

from covers import GenericCover

c = logging.getLogger(__name__)

URL_TEMPLATE = 'http://www.albumart.org/index.php?srchkey=%s+%s&searchindex=Music'

class Amazon(GenericCover):
    __title__ = 'Amazon'
    __version__ = '0.1'
    __authors__ = 'Alexandr Grigorcea <cahr.gr@gmail.com>'
    __desc__ = _('Download covers from Amazon.com')

    def get_cover(self, track):
        c.debug('Amazon cover downloader started')

        if not track.artist:
            c.warning('Amazon: ' + _('Track has empty artist field'))
            artist = ''
        else:
            artist = urllib2.quote(track.artist.lower().encode('utf-8'))
        if not track.album:
            c.warning('Amazon: ' + _('Track has empty album field'))
            album = ''
        else:
            album = urllib2.quote(track.album.lower().encode('utf-8'))

        url = URL_TEMPLATE % (artist, album)

        c.debug("Amazon url: "+url)

        content = urllib2.urlopen(url).read()

        regex = re.compile("<img src=\"(.*)\" border=\"0\" class=\"image_border\"")

        search = re.search(regex, content)
        if search is None:
            c.warning('Amazon: ' + _('Cover not found'))
            return None
        else:
            path = search.group(1)
            c.debug("Image url: "+path)

        # try to find bigger cover
        if path.find('._SL160_') != -1:
            path = path.replace('._SL160_','')

        i = path.rfind('.')
        ext = path[i:] if i > -1 else '.jpg'
        f = NamedTemporaryFile(delete=False, prefix='cg_tmp_', suffix=ext)

        try:
            f.write(urllib2.urlopen(path).read())
            f.close()
            return f.name
        except:
            c.warning('Amazon:' + _('Failed downloading cover'))
            f.close()
            os.unlink(f.name)
        return None
