# Copyright (C) 2009-2010 Gloobus Developers
#
#This program is free software; you can redistribute it and/or
#modify it under the terms of the GNU General Public License
#as published by the Free Software Foundation; either version 2
#of the License, or (at your option) any later version.
#
#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

import cStringIO
import gzip
import logging
import os
import urllib2

from tempfile import NamedTemporaryFile
from xml.dom import minidom

from gettext import gettext as _

from covers import GenericCover

c = logging.getLogger(__name__)

class Discogs(GenericCover):
    __title__ = 'Discogs'
    __version__ = '0.1'
    __authors__ = 'Alexandr Grigorcea <cahr.gr@gmail.com>'
    __desc__ = _('Download covers from Discogs.com')

    api_key = 'ef17965f90'
    artist_url = 'http://www.discogs.com/artist/{artist}?f=xml&api_key={api_key}'
    release_url = 'http://www.discogs.com/release/{id}?f=xml&api_key={api_key}'

    def get_cover(self, track):
        releases = self.get_releases(track.artist, track.album)
        for rel in releases:
            urls = self.get_cover_urls(rel)
            if urls:
                for url in urls:
                    i = url.rfind('.')
                    ext = url[i:] if i > -1 else '.jpg'
                    f = NamedTemporaryFile(delete=False, prefix='cg_tmp_', suffix=ext)
                    try:
                        f.write(urllib2.urlopen(url).read())
                        f.close()
                        return f.name
                    except:
                        f.close()
                        os.unlink(f.name)

        return None

    def get_releases( self, artist, album):
        album = album.lower()
        ids = []
        url = self.artist_url.format(artist=urllib2.quote(artist), api_key=self.api_key)
        request = urllib2.Request(url)
        request.add_header('Accept-Encoding', 'gzip')
        try:
            response = urllib2.urlopen(request)
            data = response.read()
            try:
                xml = minidom.parseString(gzip.GzipFile(fileobj = cStringIO.StringIO(data)).read())
            except:
                xml = minidom.parseString(data)

            xml = xml.getElementsByTagName('resp')[0]
            xml = xml.getElementsByTagName('artist')[0]
            xml = xml.getElementsByTagName('releases')[0]
            for release in xml.getElementsByTagName('release'):
                for title in release.getElementsByTagName('title'):
                    t = title.firstChild.data
                    if t.lower() == album:
                        ids.append(release.getAttribute('id'))
        except:
            pass
        return ids

    def get_cover_urls(self, id):
        url = self.release_url.format(id=id, api_key=self.api_key)
        request = urllib2.Request(url)
        request.add_header('Accept-Encoding', 'gzip')
        images = [None,]
        try:
            response = urllib2.urlopen(request)
            data = response.read()
            try:
                xml = minidom.parseString(gzip.GzipFile(fileobj = cStringIO.StringIO(data)).read())
            except:
                xml = minidom.parseString(data)

            xml = xml.getElementsByTagName('resp')[0]
            xml = xml.getElementsByTagName('release')[0]
            for node in xml.getElementsByTagName('images'):
                for image in node.getElementsByTagName('image'):
                    if image.getAttribute('type') == 'primary':
                        images[0] = image.getAttribute('uri')
                    else:
                        images.append(image.getAttribute('uri'))
            return images
        except:
            c.warning('Discogs: ' + _('Unable to obtain album data'))
            return None
