# -*- coding: utf8 -*-
# Copyright (C) 2010 Sun Ning <classicning@gmail.com>
#
#This program is free software; you can redistribute it and/or
#modify it under the terms of the GNU General Public License
#as published by the Free Software Foundation; either version 2
#of the License, or (at your option) any later version.
#
#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

import logging
import json
import os
import traceback
import urllib

from tempfile import NamedTemporaryFile

from gettext import gettext as _

from covers import GenericCover

c = logging.getLogger(__name__)

class Douban(GenericCover):
    __title__ = 'Douban'
    __version__ = '0.1'
    __authors__ = 'Sun Ning <classicning@gmail.com>'
    __desc__ = _('Download covers from douban.com')

    def get_cover(self, track):
        artist = track.artist or ''
        album = track.album or ''

        f = NamedTemporaryFile(delete=False, prefix='cg_tmp_', suffix='.jpg')
        try:
            g = self._search(artist+' '+album)
            url = g.next()
            ## replace with large picture
            url = url.replace('spic', 'lpic')
            data = urllib.urlopen(url).read()
            f.write(data)
            f.close()
            return f.name
        except:
#            traceback.print_exc()
            c.warning(self.__title__ + _('Failed downloading cover'))
            f.close()
            os.unlink(f.name)
            return None

    def _search(self, key, apikey=None):
        quoted = urllib.quote(key)
        urltemplate="http://api.douban.com/music/subjects?q=%s&max-results=5&alt=json"
    	query = urltemplate %(quoted)

        if apikey:
            urltemplate += ("&api_key=%s" % apikey)

        h = urllib.urlopen(query)
        data = h.read()

        result = json.loads(data)
        if(len(result['entry'])>0):
            for entry in result['entry']:
                links = entry['link']
                for link in links:
                    if link['@rel'] == 'image':
                        yield link['@href']
