# Copyright (C) 2009-2010 Gloobus Developers
#
#This program is free software; you can redistribute it and/or
#modify it under the terms of the GNU General Public License
#as published by the Free Software Foundation; either version 2
#of the License, or (at your option) any later version.
#
#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

import dbus
import logging
import os
import urllib2

from tempfile import NamedTemporaryFile
from urllib2 import quote
from urllib2 import urlopen
from xml.dom import minidom

from gettext import gettext as _

from covers import GenericCover

c = logging.getLogger(__name__)

class Lastfm(GenericCover):
    __title__ = 'Last.fm'
    __version__ = '0.1'
    __authors__ = 'Alexandr Grigorcea <cahr.gr@gmail.com>'
    __desc__ = _('Download covers from Last.fm')

    api_key = '03847dec370bfcfe516815d7ff8a3da2'
    info_url = 'http://ws.audioscrobbler.com/2.0/?method=album.getinfo&api_key={api_key}&artist={artist}&album={album}'

    def get_cover(self, track):
        url = self.info_url.format(api_key=self.api_key,
                                   artist=quote(track.artist),
                                   album=quote(track.album))
        try:
            data = urlopen(url).read()
            xml = minidom.parseString(data)
            xml = xml.getElementsByTagName('lfm')[0]
            if xml.getAttribute('status') != 'ok':
                return None
            album = xml.getElementsByTagName('album')[0]
            url = None
            for image in album.getElementsByTagName('image'):
                if image.getAttribute('size') == 'extralarge':
                    url = image.firstChild.data

            if url:
                i = path.rfind('.')
                ext = path[i:] if i > -1 else '.jpg'
                f = NamedTemporaryFile(delete=False, prefix='cg_tmp_', suffix=ext)

                try:
                    f.write(urlopen(url).read())
                    return f.name
                except:
                    c.warning('Last.fm' + _('Failed downloading cover'))
                finally:
                    f.close()
                os.unlink(f.name)
            else:
                c.warning('Last.fm: ' + _('Cover not found'))
        except Exception as e:
            c.warning('Last.fm: ' + _('Cover search error') + ' - ' + str(e))
        return None
