# Copyright (C) 2009-2010 Gloobus Developers
#
#This program is free software; you can redistribute it and/or
#modify it under the terms of the GNU General Public License
#as published by the Free Software Foundation; either version 2
#of the License, or (at your option) any later version.
#
#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

import gtk
import os
import re

from gettext import gettext as _

from core import config
from covers import GenericCover

#PATTERN = '^(folder|cover|front|album|albumart|{0} - {1})\.(jpg|jpeg|png|gif)$'
#SHORT_PATTERN = '^(folder|cover|front|album|albumart)\.(jpg|jpeg|png|gif)$'

FILENAMES = ('folder', 'cover', 'front', 'album', 'albumart')
EXTENTIONS = ('jpg', 'jpeg', 'png', 'gif')

class Location(GenericCover):
    __title__ = 'Location Search'
    __version__ = '0.1'
    __authors__ = 'Alexandr Grigorcea <cahr.gr@gmail.com>'
    __desc__ = _('Search cover in songs directory')

    __local__ = True

    __reg = None

    def __init__(self):
        super(Location, self).__init__()
        if not Location.__reg:
            Location.__make_reg()

    def get_cover(self, track):
        if track.location and Location.__reg:
            path = track.location
            if path.startswith('file://'):
                path = path[7:]
            path = os.path.dirname(path)
#            if track.artist and track.album:
#                pattern = PATTERN.format(re.escape(track.artist),
#                                         re.escape(track.album))
#            else:
#                pattern = SHORT_PATTERN
#            reg = re.compile(pattern, re.I)
            for name in os.listdir(path):
                if Location.__reg.match(name):
                    return path + '/' + name
        return None

    @staticmethod
    def configure(dialog):
        vbox = dialog.get_content_area()
        vbox.set_border_width(10)
        LABEL_WIDTH = 120
        ENTRY_WIDTH = 200

        values = config.get('Covers', 'location_filenames').split(',')
        filenames = []
        for name in FILENAMES:
            check = gtk.CheckButton(name)
            filenames.append(check)
            if name in values:
                check.set_active(True)

        label = gtk.Label('<b>' + _('Search filenames') + ':</b>')
        label.set_use_markup(True)
        label.set_alignment(0.0, 0.0)
        table = gtk.Table(3, 2, True)
        table.attach(filenames[0], 0, 1, 0, 1)
        table.attach(filenames[1], 0, 1, 1, 2)
        table.attach(filenames[2], 0, 1, 2, 3)
        table.attach(filenames[3], 1, 2, 0, 1)
        table.attach(filenames[4], 1, 2, 1, 2)
        vbox.pack_start(label, False)
        vbox.pack_start(table, False)

#        box = gtk.HBox(False, 10)
#        label = gtk.Label(_('Other filenames') + ':')
#        label.set_alignment(0.0, 0.5)
#        other_names = gtk.Entry()
#        other_names.set_text(','.join(names))
#        box.pack_start(label, False)
#        box.pack_start(other_names, True)
#        vbox.pack_start(box, False)

        values = config.get('Covers', 'location_extentions').split(',')
        extentions = []
        for name in EXTENTIONS:
            check = gtk.CheckButton(name)
            extentions.append(check)
            if name in values:
                check.set_active(True)

        label = gtk.Label('<b>' + _('Search extentions') + ':</b>')
        label.set_use_markup(True)
        label.set_alignment(0.0, 0.0)
        table = gtk.Table(2, 2, True)
        table.attach(extentions[0], 0, 1, 0, 1)
        table.attach(extentions[1], 0, 1, 1, 2)
        table.attach(extentions[2], 1, 2, 0, 1)
        table.attach(extentions[3], 1, 2, 1, 2)
        vbox.pack_start(label, False)
        vbox.pack_start(table, False)

        vbox.show_all()

        if dialog.run() == gtk.RESPONSE_ACCEPT:
            values = [f.get_label() for f in filenames if f.get_active()]
            config.set('Covers', 'location_filenames', ','.join(values))
            values = [f.get_label() for f in extentions if f.get_active()]
            config.set('Covers', 'location_extentions', ','.join(values))
            Location.__make_reg()

    @staticmethod
    def __make_reg():
        filenames = [f.strip() for f in config.get('Covers', 'location_filenames').split(',')]
        extentions = [f.strip() for f in config.get('Covers', 'location_extentions').split(',')]

        if len(filenames) > 0 and len(extentions) > 0:
            pattern = '^(' + '|'.join(filenames) + ')\.(' + '|'.join(extentions) + ')$'
            Location.__reg = re.compile(pattern, re.I)
        else:
            Location.__reg = None
