# Copyright (C) 2009-2010 Gloobus Developers
#
#This program is free software; you can redistribute it and/or
#modify it under the terms of the GNU General Public License
#as published by the Free Software Foundation; either version 2
#of the License, or (at your option) any later version.
#
#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

import gobject
import logging

from subprocess import Popen, PIPE

from gettext import gettext as _

from core.track import State
from core.track import Track
from core.common import COVER, STATE, TIME, TRACK
from generic.Cli import Cli

c = logging.getLogger(__name__)

class DeaDBeeF(Cli):

    __title__ = 'DeaDBeeF'
    __version__ = '0.1'
    __authors__ = 'Alexandr Grigorcea <cahr.gr@gmail.com>'
    __desc__ = _('DeaDBeeF plugin')

    _data = ''
    _state = State.STOPPED
    _track = Track()

    def update(self):
        p = Popen(['deadbeef', '--nowplaying','%t\t%b\t%a\t%n\t%y\t%l\t%g\t%F'], stdout=PIPE, stderr=PIPE)
        data = p.communicate()[0]

        if data == 'nothing':
            if self._state != State.STOPPED:
                self._state = State.STOPPED
                self.cg.update(STATE, State.STOPPED)
        else:
            if self._state != State.PLAYING:
                self._state = State.PLAYING
                self.cg.update(STATE, State.PLAYING)

            if data != self._data:
                self._data = data
                self._track = self._get_track(data)
                self.cg.update(TRACK, self._track)

        return True

    def get_pid(self):
        return Popen(['pidof', 'deadbeef'], stdout=PIPE).communicate()[0].strip()

    def _get_track(self, string):
        vals = string.split('\t')
        data = {}
        i = 0
        for key in ('title', 'album', 'artist', 'tracknumber', 'year', 'length', 'genre', 'location'):
            try:
                data[key] = vals[i]
                i += 1
            except:
                pass

        return Track(**data)


    def get_track(self):
        return self._track

    def get_state(self, update=True):
        return self._state

    def previous(self):
        Popen(['deadbeef', '--prev'])

    def next(self):
        Popen(['deadbeef', '--next'])

    def play(self):
        Popen(['deadbeef', '--play-pause'])
