# Copyright (C) 2009-2010 Gloobus Developers
#
#This program is free software; you can redistribute it and/or
#modify it under the terms of the GNU General Public License
#as published by the Free Software Foundation; either version 2
#of the License, or (at your option) any later version.
#
#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

import gobject
import logging
import os

from gettext import gettext as _

from core.track import State
from core.track import Track
from core.common import COVER, STATE, TIME, TRACK
from core.common import HOME_PATH
from generic.Player import Player

c = logging.getLogger(__name__)

GF = 'GroovesharkDesktop.7F9BF17D6D9CB2159C78A6A6AB076EA0B1E0497C.1'
STATE_PATH = '%s/Documents/Grooveshark/currentSong.txt' % HOME_PATH
CONTROL_PATH = '%s/.appdata/%s/Local Store/shortcutAction.txt' % (HOME_PATH, GF)

TITLE, ALBUM, ARTIST, STATUS = range(4)

class Grooveshark(Player):

    __title__ = 'Grooveshark'
    __version__ = '0.1'
    __authors__ = 'Alexandr Grigorcea <cahr.gr@gmail.com>'
    __desc__ = _('Grooveshark desktop player plugin')

    _control = False
    _last_mtime = 0
    _last_track = Track()
    _last_state = 'closed'
    _timer_id = 0

    def is_active(self):
        return os.path.exists(STATE_PATH)

    def connect(self):
        if not self._timer_id:
            self._timeout_id = gobject.timeout_add(2000, self.check_new_song)
            self.check_new_song()
        if os.path.exists(CONTROL_PATH):
            self._control = True
        self.connected = True
        return True

    def disconnect(self):
        if self._timer_id:
            gobject.source_remove(self._timeout_id)
        self.connected = False

    def check_new_song(self):
        mtime = os.stat(STATE_PATH).st_mtime
        if self._last_mtime < mtime:
            self._last_mtime = mtime
            fd = open(STATE_PATH)
            content = fd.read().split("\t")
            fd.close()

            try:
                if self._last_track.title != content[TITLE] or self._last_track.album != content[ALBUM] or self._last_track.artist != content[ARTIST]:
                    self._last_track = Track(
                        title = content[TITLE],
                        album = content[ALBUM],
                        artist = content[ARTIST]
                    )
                    self.cg.update(TRACK, self._last_track)
            except:
                pass

            try:
                if self._last_state != content[STATUS]:
                    self.cg.update(STATE, State.get(content[STATUS]))
                    self._last_state = content[STATUS]
                    if self._last_state == 'stopped':
                        self._last_track = Track()
            except:
                pass
        return True

    def get_track(self):
        return self._last_track

    def get_state(self):
        return State.get(self._last_state)

    def previous(self):
        if self._control:
            fd = open(CONTROL_PATH, 'a')
            fd.write('previous \n')
            fd.close()

    def play(self):
        if self._control:
            fd = open(CONTROL_PATH, 'a')
            fd.write('playpause\n')
            fd.close()

    def next(self):
        if self._control:
            fd = open(CONTROL_PATH, 'a')
            fd.write('next\n')
            fd.close()
