#This program is free software; you can redistribute it and/or
#modify it under the terms of the GNU General Public License
#as published by the Free Software Foundation; either version 2
#of the License, or (at your option) any later version.
#
#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

import dbus
import gobject
import logging

from gettext import gettext as _

from core.track import State
from core.track import Track
from core.common import TRACK, STATE
from generic.Player import Player

c = logging.getLogger(__name__)

class Listen(Player):
    __title__ = 'Listen'
    __version__ = '0.0'
    __authors__ = 'Kamencesc <kamencesc@gmail.com>, Alexandr Grigorcea <cahr.gr@gmail.com>'
    __desc__ = _('Listen music player plugin')

    DBUS_NAME   = 'org.gnome.Listen'
    DBUS_OBJECT = '/org/gnome/listen'

    capabilities = ('time',)

    iface = None

    _data = {}
    _state = State.CLOSED
    _timeout_id = None
    _track = Track()

    def connect(self):
        session_bus = dbus.SessionBus()
        self.iface = session_bus.get_object(self.DBUS_NAME, self.DBUS_OBJECT)
        self._timeout_id = gobject.timeout_add(2000, self.check_new_song)
        self.connected = True
        return True

    def disconnect(self):
        if self._timeout_id:
            gobject.source_remove(self._timeout_id)
        self.connected = False

    def check_new_song(self):
        state = self.get_state()
        if state != self._state:
            self._state = state
            self.cg.update(STATE, state)
        if state == State.PLAYING:
            data = {
                'title' : self.iface.get_title(),
                'album' : self.iface.get_album(),
                'artist' : self.iface.get_artist(),
                'cover' : self.iface.get_cover_path(),
                'location' : self.iface.get_uri(),
                'length' : self.iface.current_song_length()
            }
            if data != self._data:
                self._data = data
                self._track = Track(**data)
                self.cg.update(TRACK, self._track)
        return True

    def get_state(self):
        if self.iface.playing():
            return State.PLAYING
        else:
            return State.PAUSED

    def get_track(self):
        return self._track

    def get_position(self):
        return self.iface.current_position()

    def previous(self):
        self.dbus_object.previous()

    def play(self):
        self.iface.play_pause()

    def next(self):
        self.dbus_object.next()
