# Copyright (C) 2009-2010 Gloobus Developers
#
#This program is free software; you can redistribute it and/or
#modify it under the terms of the GNU General Public License
#as published by the Free Software Foundation; either version 2
#of the License, or (at your option) any later version.
#
#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

import gobject
import logging

from subprocess import Popen, PIPE

from gettext import gettext as _

from core.track import State
from core.track import Track
from core.common import TRACK, STATE
from generic.Cli import Cli

c = logging.getLogger(__name__)

class MOC(Cli):
    __title__ = 'MOC'
    __version__ = '0.1'
    __authors__ = 'Alexandr Grigorcea <cahr.gr@gmail.com>'
    __desc__ = _('MOC (Music On Console) plugin')

    capabilities = ('time', 'seek')

    _data = {}
    _state = State.CLOSED

    def update(self):
        p = Popen(['mocp', '--info'], stdout=PIPE)
        output = p.communicate()[0].strip().split("\n")

        try:
            state = output[0].split(': ')[1]
        except:
            return True

        if state == 'PLAY':
            state = State.PLAYING
        elif state == 'PAUSE':
            state = State.PAUSED
        elif state == 'STOP':
            state = State.STOPPED
        else:
            # a warning?
            return True

        if state != self._state:
            self._state = state
            self.cg.update(STATE, state)

        if state != State.STOPPED:
            info = {}
            for line in output[1:]:
                key, val = line.split(': ')
                info[key] = val

            data = {
                'title': info['SongTitle'],
                'album': info['Album'],
                'artist': info['Artist'],
                'length': info['TotalSec'],
                'location': info['File']
            }
            if data != self._data:
                self._data = data
                self.cg.update(TRACK, Track(**data))
        return True

    def get_track(self):
        return Track(**self._data)

    def get_state(self, update=True):
        return self._state

    def get_position(self):
        p = Popen(['mocp', '--info'], stdout=PIPE)
        output = p.communicate()[0].strip().split("\n")

        if len(output) > 10:
            try:
                return int(output[10].split(': ')[1])
            except:
                pass
        return 0

    def get_pid(self):
        return Popen(['pidof', 'mocp'], stdout=PIPE).communicate()[0].strip()

    def set_position(self, time):
        sec = time - self.get_position()
        Popen(['mocp', '--seek', str(sec)])

    def previous(self):
        Popen(['mocp', '--previous'])

    def next(self):
        Popen(['mocp', '--next'])

    def play(self):
        Popen(['mocp', '--toggle-pause'])
