# Copyright (C) 2009-2010 Gloobus Developers
#
#This program is free software; you can redistribute it and/or
#modify it under the terms of the GNU General Public License
#as published by the Free Software Foundation; either version 2
#of the License, or (at your option) any later version.
#
#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

import dbus
import logging

from gettext import gettext as _

from core.track import State
from core.track import Track
from core.common import COVER, STATE, TIME, TRACK
from core.common import HOME_PATH
from generic.Player import Player

c = logging.getLogger(__name__)

class Pithos(Player):
    __title__ = 'Pithos'
    __version__ = '0.1'
    __authors__ = 'Alexandr Grigorcea <cahr.gr@gmail.com>'
    __desc__ = _('Pithos player plugin')

    DBUS_NAME   = 'net.kevinmehall.Pithos'
    DBUS_OBJECT = '/net/kevinmehall/Pithos'

    _player = None
    _signals = []

    def connect(self):
        try:
            if not self.connected:
                proxy = dbus.SessionBus().get_object(self.DBUS_NAME, self.DBUS_OBJECT)
                self._player = dbus.Interface(proxy, self.DBUS_NAME)

                self._signals.append(self._player.connect_to_signal(
                    'PlayStateChanged', self.state_changed))
                self._signals.append(self._player.connect_to_signal(
                    'SongChanged', self.song_changed))
                self.connected = True
        except:
            self.disconnect()
        return self.connected

    def disconnect(self):
        for signal in self._signals:
            signal.remove()

        self._player = None
        self._signals = []
        self.connected = False

    def get_state(self, data=None):
        """
            Returns players actual state.
            @return: string - playing, paused, stopped
        """
        if data is None:
            data = self._player.IsPlaying()
        if data:
            return State.PLAYING
        else:
            return State.PAUSED

    def get_track(self, data=None):
        """
            Prepares a Track object with current playing song data

            @param data: dictionary, contains info about current playing song.
                optional
            @return: Track, object with current playing song data
        """
        if not data:
            data = self._player.GetCurrentSong()

        vals = ('title','album','artist')
        for key in [key for key in vals if key not in data]:
            c.warning('%s: %s not found in current track' % ('PITHOS', key))
            data[key] = ''

        track = Track(
            title = data['title'],
            album = data['album'],
            artist= data['artist']
        )
        return track

    def previous(self):
        self._player.SkipSong()

    def play(self):
        self._player.PlayPause()

    def next(self):
        self._player.SkipSong()

# ================================= HANDLERS ================================= #

    def state_changed(self, message):
        self.cg.update(STATE, self.get_state(message))

    def song_changed(self, message):
        self.cg.update(TRACK, self.get_track(message))
