# Copyright (C) 2009-2010 Gloobus Developers
#
#This program is free software; you can redistribute it and/or
#modify it under the terms of the GNU General Public License
#as published by the Free Software Foundation; either version 2
#of the License, or (at your option) any later version.
#
#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.


from gettext import gettext as _

from core.common import HOME_PATH
from generic.Mpris import Mpris

class Songbird(Mpris):

    __title__ = 'Songbird'
    __version__ = '0.1'
    __authors__ = 'Jaap Broekhuizen <jaapz.b@gmail.com>, Alexandr Grigorcea <cahr.gr@gmail.com>'
    __desc__ = _('Songbird music player plugin. Requires MPRIS plugin for Songbird.')

    DBUS_NAME   = 'org.mpris.songbird'

    _artwork_path = None

    capabilities = ('rating', 'time', 'seek', 'volume')

    def get_track(self, data=None):
        track = super(SongbirdPlayer, self).get_track(data)
        if track.cover[:21] == 'resource://sb-artwork':
            track.cover = self._get_artwork_path() + track.cover[21:]
        return track

    def set_rating(self,rating):
        if 0 <= rating and rating <= 5:
            self.dbus_object.SetMetadata('rating', str(rating))

    def _get_artwork_path(self):
        if self._artwork_path:
            return self._artwork_path

        profile = open('%s/.songbird2/profiles.ini' % HOME_PATH)

        found = False
        relative = False
        name = None
        for line in profile:
            line = line.strip()
            data = line.split('=',1)
            if len(data) == 2:
                key = data[0].strip()
                val = data[1].strip()
                if key == 'Name' and val == 'default':
                    found = True
                if found and key == 'Path':
                    name = val
                if found and key == 'IsRelative':
                    if val == '1':
                        relative = True
        if name:
            if relative:
                self._artwork_path = '%s/.songbird2/%s/artwork'%(HOME_PATH,name)
            else:
                if self._artwork_path[1:] == '/':
                    self._artwork_path = self._artwork_path[:-1]
                self._artwork_path = name + '/artwork/'
            return self._artwork_path
