# Copyright (C) 2009-2010 Gloobus Developers
#
#This program is free software; you can redistribute it and/or
#modify it under the terms of the GNU General Public License
#as published by the Free Software Foundation; either version 2
#of the License, or (at your option) any later version.
#
#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

# Spotify Plugin
# From spotify-notify by SveinT (sveint@gmail.com)
# Original code by noalwin (lambda512@gmail.com)
# Based on pypanel code
# Developed by BadChoice (jordi.hernandezp@gmail.com)

import os, time, sys, datetime, string
import logging
import urllib2
import re
import gobject

from gettext import gettext as _

from core.track import State
from core.track import Track
from core.common import TRACK, STATE
from core.utils import wikilyrics_get_album

from generic.Player import Player

c = logging.getLogger(__name__)

try:
    from Xlib import X, display, error, Xatom
    import Xlib.protocol.event
except:
    c.warning(_("You need the xlib-library. If you are using Ubuntu, try running\n\"sudo apt-get install python-xlib\""), True)
    sys.exit(1)

CURRENT_DIR = os.path.abspath(os.curdir).replace(';','')+"/"


class Spotify(Player):

    __title__ = 'Spotify'
    __version__ = '0.0'
    __authors__ = 'BadChoice <guitarboy000@gmail.com>'
    __desc__ = _('Spotify music player plugin')

    timeout_id = None
    track = Track()

    _last_title = ''
    _last_artist = ''
    _last_album = ''
    _last_state = State.STOPPED

    def __init__(self, cg):
        super(Spotify, self).__init__(cg)
        self._dsp = display.Display()
        self._screen  = self._dsp.screen()
        self._root    = self._screen.root
    #   self.error   = error.CatchError()
        self._root.change_attributes(event_mask=(X.PropertyChangeMask))
        self._dsp.flush()


        self._spotify_task = None
        self._spotify_obj = None
        self._spotify_title = None # Spotify's window title

        self._init_spotify_data()

    def _init_spotify_data(self):
        """Search the spotify window and initialize the data"""
        tasks = self._root.get_full_property(
                        self._dsp.intern_atom("_NET_CLIENT_LIST"),
                        Xatom.WINDOW
                ).value

        is_running = False

        for task in tasks:
            obj  = self._dsp.create_resource_object("window", task)
            name = ( obj.get_full_property(self._dsp.intern_atom("_NET_WM_NAME"), 0) or
                     obj.get_full_property(Xatom.WM_NAME, 0) )
            title = getattr(name,'value','')
            if title is not '' and title.startswith('Spotify'):
                obj.change_attributes(event_mask=(
                    X.PropertyChangeMask|X.StructureNotifyMask))
                is_running = True
                self._spotify_task = task
                self._spotify_obj = obj
                self._spotify_title = title
                break

        if not is_running and self._spotify_task is not None:
            self._delete_spotify_data()


    def _update_spotify_title(self):
        name = ( self._spotify_obj.get_full_property(self._dsp.intern_atom("_NET_WM_NAME"), 0) or
                 self._spotify_obj.get_full_property(Xatom.WM_NAME, 0) )
        title = getattr(name,'value','')
        if title is not '' and title.startswith('Spotify'):
            self._spotify_title = title

    def _delete_spotify_data(self):
        self._spotify_task = None
        self._spotify_obj = None
        self._spotify_title = None


    def _update(self):
        while self._dsp.pending_events():
            e = self._dsp.next_event()
            #print e

            if e.type == X.DestroyNotify:
                if e.window.id == self._spotify_task:
                    self._delete_spotify_data()
                    #print "Spotify has been closed"
            elif e.type == X.PropertyNotify:
                if e.atom == self._dsp.intern_atom("_NET_CLIENT_LIST"):
                    #print '_NET_CLIENT_LIST'
                    if self._spotify_task is not None:
                        #if hasattr(e, "window") and self._spotify_task == e.window.id:
                            #print 'Spotify seems to have changed'
                        pass
                    else:
                        # print 'Spotify has been started'
                        self._init_spotify_data()
                elif self._spotify_task is not None and e.window.id == self._spotify_task:
                    if e.atom in [Xatom.WM_NAME, self._dsp.intern_atom("_NET_WM_NAME")]:
                        try:
                            self._update_spotify_title()
                            #print 'Spotify has changed the name %s' % self._spotify_title
                        except Xlib.error.BadWindow:
                            #print "BADWINDOW!"
                            pass

    def get_song(self):
        self._update()
        # self._spotify_title -> "Spotify \xe2\x80\x93 Title - Artist"

        if self._spotify_title is None or self._spotify_title == 'Spotify':
            # Closed or stopped
            return None, None

        data = self._spotify_title[10:].split(' \xe2\x80\x93 ')
        #print self._spotify_title
        #data = self._spotify_title[10:].split('\x93 ')

        if len(data) == 2:
        #print "A: " + data[0]
        #print "T: " + data[1]
            #return { 'artist': data[0], 'title': data[1] }
            return data
        else:
            #Unexpected format
            #return { 'artist':'', 'title': self._spotify_title[10:] }
            return '', self._spotify_title[10:]

    def _get_album(self, artist, title):
        if title != self._last_title or artist != self._last_artist:
            self._last_album = wikilyrics_get_album(artist, title)
        return self._last_album


    def is_active(self):
        return True
        #self._update()
        #if self._spotify_title:
        #    return True
        #else:
        #    return False

    def connect(self):
        if not self.timeout_id:
            self.timeout_id = gobject.timeout_add(2000, self.check_new_song)
        self.connected = True
        return True

    def disconnect(self):
        if self.timeout_id:
            gobject.source_remove(self.timeout_id)
        self.connected = False

    def check_new_song(self):
        artist, title = self.get_song()
        if not artist and not title:
            state = State.STOPPED
        else:
            state = State.PLAYING

        if self._last_state != state:
                self._last_state = state
                self.cg.update(STATE, state)

        if self._last_state != State.STOPPED:
            if title != self._last_title or artist != self._last_artist:
                self._last_title = title
                self._last_artist = artist
                self._last_album = wikilyrics_get_album(artist, title)
                self.cg.update(TRACK,
                                Track(
                                    title = self._last_title,
                                    artist = self._last_artist,
                                    album = self._last_album
                                ))
        return True

    def get_track(self):
        return Track()

    def get_state(self):
        return State.PLAYING
