# Copyright (C) 2009-2010 Gloobus Developers
#
#This program is free software; you can redistribute it and/or
#modify it under the terms of the GNU General Public License
#as published by the Free Software Foundation; either version 2
#of the License, or (at your option) any later version.
#
#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

import gobject
import logging
import os

from gettext import gettext as _

from core.track import State
from core.track import Track
from core.common import COVER, STATE, TIME, TRACK
from core.common import ABS_PATH

from Player import Player

c = logging.getLogger(__name__)

class Cli(Player):
    COMMAND = None
    OBSERVE_TIMEOUT = 5000
    UPDATE_TIMEOUT = 2000

    _observe_command_id = 0
    _observe_pid_id = 0
    _timeout_id = 0

    _pid = None

    def is_active(self):
        self._pid = self.get_pid()
        if self._pid:
            return True
        else:
            self._observe_command_id = gobject.timeout_add(self.OBSERVE_TIMEOUT,
                                                           self._observe_command)
            return False

    def _observe_command(self):
        self._pid = self.get_pid()
        if self._pid:
            if self.connect():
                self.cg.update(STATE, State.STARTED)
                self._observe_pid_id = gobject.timeout_add(self.OBSERVE_TIMEOUT,
                                                           self._observe_pid)
                self._observe_command_id = 0
                return False
        return True

    def _observe_pid(self):
        if not os.path.exists(os.path.join('/proc', self._pid)):
            self.disconnect()
            self.cg.update(STATE, State.CLOSED)
            self._observe_command_id = gobject.timeout_add(self.OBSERVE_TIMEOUT,
                                                           self._observe_command)
            self._observe_pid_id = 0
            return False
        return True

    def connect(self):
        self._timeout_id = gobject.timeout_add(self.UPDATE_TIMEOUT, self.update)
        self.update()
        self.connected = True
        return True

    def disconnect(self):
        if self._timeout_id:
            gobject.source_remove(self._timeout_id)
            self._timeout_id = 0
        self.connected = False

    def update(self):
        raise NotImplementedError

    def get_pid(self):
        raise NotImplementedError
