# Copyright (C) 2009-2010 Gloobus Developers
#
#This program is free software; you can redistribute it and/or
#modify it under the terms of the GNU General Public License
#as published by the Free Software Foundation; either version 2
#of the License, or (at your option) any later version.
#
#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

import dbus
import gobject
import logging
import os

from gettext import gettext as _

from core.track import State
from core.track import Track
from core.common import TRACK, STATE
from generic.Player import Player

c = logging.getLogger(__name__)

class gmusicbrowser(Player):

    __title__ = 'gmusicbrowser'
    __version__ = '0.1'
    __authors__ = 'Alexandr Grigorcea <cahr.gr@gmail.com>'
    __desc__ = _('gmusicbrowser jukebox plugin')

    capabilities = ('rating', 'time')

    DBUS_NAME   = 'org.gmusicbrowser'
    DBUS_OBJECT = '/org/gmusicbrowser'

    current_id = None
    iface = None
    state = 0
    timeout_id = 0

    def connect(self):
        """
            Connects player
            @return: boolean, True if successful, False if not
        """
        session_bus = dbus.SessionBus()
        self.iface = session_bus.get_object(self.DBUS_NAME,
            self.DBUS_OBJECT)
        if not self.connected:
            session_bus.add_signal_receiver(self.song_changed_handler,
                "SongChanged", 'org.gmusicbrowser',
                self.DBUS_NAME, self.DBUS_OBJECT)
            self.timeout_id = gobject.timeout_add(2000, self.check_state)
            self.connected = True
        return True

    def disconnect(self):
        """
            Disconnects player
        """
        if self.connected:
            session_bus = dbus.SessionBus()
            session_bus.remove_signal_receiver(self.song_changed_handler,
                "SongChanged", 'org.gmusicbrowser',
                self.DBUS_NAME, self.DBUS_OBJECT)
            self.connected = False
            if self.timeout_id:
                gobject.source_remove(self.timeout_id)

    def check_state(self):
        state = self.iface.Playing()
        if state != self.state:
            self.state = state
            if state:
                state = State.PLAYING
            else:
                state = State.PAUSED
            self.cg.update(STATE, state)
        return True

    def get_rating(self):
        try:
            rating = str(self.get_tag('rating'))
            return (int(rating)/20)
        except:
            return 0

    def get_state(self):
        if self.iface.Playing():
            return State.PLAYING
        else:
            return State.PAUSED

    def previous(self):
        self.iface.RunCommand('PrevSong')

    def play(self):
        self.iface.RunCommand('PlayPause')

    def next(self):
        self.iface.RunCommand('NextSong')

    def get_track(self):
        data = self.iface.CurrentSong()

        vals = ('title','album','artist', 'track','path', 'album_picture')
        for key in [key for key in vals if key not in data]:
            c.warning('%s: %s not found in current track' % ('GMUSICBROWSER', key))
            data[key] = ''

        track = Track(
            title  = data['title'],
            album  = data['album'],
            artist = data['artist'],
            genre  = self.get_tag('genre'),
            year   = self.get_tag('date'),
            tracknumber = data['track'],
            location = os.path.join(data['path'], data['file']),
            rating = self.get_rating(),
            cover  = data['album_picture'],
            length = data['length']
        )
        return track

    def get_position(self):
        return int(self.iface.GetPosition())

    def set_rating(self,rating):
        if 0 <= rating and rating <= 5:
            rating = rating*20;
            self.set_tag('rating',str(rating))

# ================================= HANDLERS ================================= #

    def song_changed_handler(self, message):
        try:
            self.current_id = int(message)
        except:
            self.current_id = None
        self.cg.update(TRACK, self.get_track())

# ================================= HELPERS ================================== #

    def get_tag(self, name):
        try:
            if self.current_id:
                id = str(self.current_id)
            else:
                data = self.iface.CurrentSong()
                id = os.path.join(data['path'], data['file'])
            return self.iface.Get((id,name))
        except:
            return ''

    def set_tag(self, name,val):
        try:
            if self.current_id:
                id = str(self.current_id)
            if self.current_id:
                data = self.iface.CurrentSong()
                id = os.path.join(data['path'], data['file'])
            self.iface.Set((id,name,val))
        except:
            return
