# Copyright (C) 2009-2010 Gloobus Developers
#
#This program is free software; you can redistribute it and/or
#modify it under the terms of the GNU General Public License
#as published by the Free Software Foundation; either version 2
#of the License, or (at your option) any later version.
#
#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

import gtk

class Rectangle(gtk.gdk.Rectangle):
    def __init__(self, x=0, y=0, width=0, height=0):
        super(Rectangle, self).__init__(x, y, width, height)
        self.src = gtk.gdk.Rectangle(x, y, width, height)

    def scale(self, percent):
        percent = float(percent)
        self.x = int(float(self.src.x) / 100.0 * percent)
        self.y = int(float(self.src.y) / 100.0 * percent)
        self.width = int(  float(self.src.width)  / 100.0 * percent)
        #if self.width < 1: self.width = 1
        self.height = int( float(self.src.height) / 100.0 * percent)
        #if self.height < 1: self.height = 1

    def set_x(self, x):
        self.x = x
        self.src.x = x

    def set_y(self, y):
        self.y = y
        self.src.y = y

    def set_width(self, width):
        self.width = width
        self.src.width = width

    def set_height(self, height):
        self.height = height
        self.src.height = height

class Reflection:
    gap = 0
    height = 0
    alpha = 1.0
    resize = 1.0

    _src_gap = 0
    _src_height = 0
    _src_resize = 0

    def load(self, str):
        ref = [0,0,0,0]
        i = 0
        try:
            for c in str:
                if c == '-':
                    i += 1
                elif not c.isdigit():
                    c.error('Reflection syntax error')
                    return False
                else:
                    ref[i] = ref[i]*10 + int(c)
                if i == 4:
                    break
        except:
            c.error('Reflection syntax error')
            return False
        if not ref[1]:
            c.error('Reflection syntax error')
            return False

        if not ref[2]  or ref[1] > 100:
            ref[2] = 0.5
        else:
            ref[2] = ref[2] / 100.0

        if not ref[3]:
            ref[3] = 1.0
        else:
            ref[3] = ref[3] / 100.0

        self.gap = ref[0]
        self._src_gap = ref[0]
        self.height = ref[1]
        self._src_height = ref[1]
        self.alpha = ref[2]
        self.resize = ref[3]
        self._src_resize = ref[3]

        return True

    def scale(self, percent):
        percent = float(percent)
        self.gap = int(float(self._src_gap) / 100.0 * percent)
        self.height = int(float(self._src_height) / 100.0 * percent)
        if self.height < 1: self.height = 1


class GenericWidget(gtk.EventBox):
    _size = None
    _reflect = None

    def __init__(self, size):
        super(GenericWidget, self).__init__()
        self._size = size
        self.set_visible_window(False)
        self.set_size_request(size.width, size.height)

    def get_x(self):
        return self._size.x

    def get_y(self):
        return self._size.y

    def get_width(self):
        return self._size.width

    def get_height(self):
        return self._size.height

    def set_reflection(self, reflect):
        self._reflect = reflect
        self.set_size_request(self.get_width(), self.get_height())
        self.queue_draw()

