# Copyright (C) 2009-2010 Gloobus Developers
#
#This program is free software; you can redistribute it and/or
#modify it under the terms of the GNU General Public License
#as published by the Free Software Foundation; either version 2
#of the License, or (at your option) any later version.
#
#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

import cairo
import gobject
import gtk
import logging
import os

from gtk.gdk import pixbuf_new_from_file_at_scale
from gtk.gdk import pixbuf_get_file_info

from widgets import GenericWidget

c = logging.getLogger(__name__)

STARS = 5 # number of start

class Rating(GenericWidget):
    __gsignals__ = {
        'set_rating': (gobject.SIGNAL_RUN_LAST, gobject.TYPE_NONE, (int,)),
    }

    _star = None
    _nostar = None
    _hover = None
    _pressed = None

    _star_path = None
    _nostar_path = None
    _hover_path = None
    _pressed_path = None

    _spacing = 0
    _src_spacing = 0
    _rating = 0
    _old_rating = 0

    _hovered = False

    _reflect = None

    _clickable = False

    def __init__(self, size, spacing, star, nostar, hover, pressed):
        super(Rating,self).__init__(size)
        if spacing > 0:
            self._spacing = spacing
            self._src_spacing = spacing

        if star and os.path.exists(star):
            self._star_path = star
        elif star:
            c.error("Rating: file not found: " + star)
        if nostar and os.path.exists(nostar):
            self._nostar_path = nostar
        elif nostar:
            c.error("Rating: file not found: " + nostar)
        if hover and os.path.exists(hover):
            self._hover_path = hover
        elif hover:
            c.error("Rating: file not found: " + hover)
        if pressed and os.path.exists(pressed):
            self._pressed_path = pressed
        elif pressed:
            c.error("Rating: file not found: " + pressed)

        self._current = self._star

        self._hovered = False

        #self.connect("motion_notify_event", self.motion_notify_event)
        self.set_events(gtk.gdk.POINTER_MOTION_MASK
            | gtk.gdk.POINTER_MOTION_HINT_MASK)

    def _load(self):
        if self._star_path:
            self._star = pixbuf_new_from_file_at_scale(self._star_path,
                                                       self._size.width,
                                                       self._size.height,
                                                       False)
        if self._nostar_path:
            self._nostar = pixbuf_new_from_file_at_scale(self._nostar_path,
                                                         self._size.width,
                                                         self._size.height,
                                                         False)
        if self._hover_path:
            self._hover = pixbuf_new_from_file_at_scale(self._hover_path,
                                                        self._size.width,
                                                        self._size.height,
                                                        False)
        if self._pressed_path:
            self._pressed = pixbuf_new_from_file_at_scale(self._pressed_path,
                                                          self._size.width,
                                                          self._size.height,
                                                          False)
        self._current = self._star
        self.set_size_request(self.get_width(), self.get_height())
        self.queue_draw()

    def do_expose_event(self, event):
        raise NotImplementedError

    def do_button_press_event(self, event):
        if self._pressed and self._clickable:
            self._current = self._pressed
            self.queue_draw()
        return True

    def do_button_release_event(self, event):
        if self._clickable:
            rating = self.check_rating(event)
            self.set_rating(rating)
            self.emit('set_rating', rating)
            if self._pressed:
                if self._hovered:
                    self._current = self._hover
                else:
                    self._current = self._star
                self.queue_draw()
        return True

    def do_enter_notify_event(self, event):
        self._hovered = True
        if self._hover and self._clickable:
            self._current = self._hover
            self.queue_draw()
        return True

    def do_leave_notify_event(self, event):
        self._hovered = False
        if self._hover and self._clickable:
            self._current = self._star
            self.queue_draw()
        self.set_rating(self._old_rating)
        return True

    def do_motion_notify_event(self, event):
        if self._clickable:
            self.set_rating(self.check_rating(event), True)
        return True

    def set_rating(self, rating, tmp=False):
        if not tmp:
            self._old_rating = rating
        self._rating = rating
        self.queue_draw()

    def check_rating(self, event):
        raise NotImplementedError

    def scale(self, percent):
        self._size.scale(percent)
        self._spacing = int( self._src_spacing / 100.0 * percent )
        if self._reflect:
            self._reflect.scale(percent)
        self._load()

        #self.star.resize(percent, False)
        #self.nostar.resize(percent, False)
        #if self.hover: self.hover.resize(percent, False)
        #if self.pressed: self.pressed.resize(percent, False)

        #self.w = self.get_width()
        #self.h = self.get_height()
        #self.set_size_request(self.w, self.h)

    def set_clickable(self, clickable):
        self._clickable = clickable

class HRating(Rating):
    def do_expose_event(self, event):
        cr = self.window.cairo_create()
        cr.rectangle(event.area.x, event.area.y,
                     event.area.width, event.area.height)
        cr.clip()
        w = 0
        for i in range(1, STARS + 1):
            if i <= self._rating:
                pix = self._current
            else:
                pix = self._nostar
            cr.set_source_pixbuf(pix, self._size.x + w, self._size.y)
            cr.paint()
            w += self._size.width + self._spacing
        # TODO draw reflection
        return True

    def get_width(self):
        return (self._size.width * STARS) + (self._spacing * (STARS - 1))

    def get_height(self):
        if self._reflect:
            return self._size.height + self._reflect.gap + self._reflect.height
        else:
            return self._size.height

    def check_rating(self, event):
        rating = 0
        w = 0
        while event.x > w and rating < STARS:
            rating += 1
            w += self._spacing + self._size.width
        return rating

class VRating(Rating):
    def do_expose_event(self, event):
        cr = self.window.cairo_create()
        cr.rectangle(event.area.x, event.area.y,
                     event.area.width, event.area.height)
        cr.clip()
        cr.translate(self._size.x, self._size.y)
        h = 0

        img = None
        for i in range(STARS, 0, -1):
            if i <= self._rating:
                img = self._current
            else:
                img = self._nostar
            cr.set_source_pixbuf(img, 0, h)
            cr.paint()
            h += self._size.height + self._spacing

        # draw reflection
        if self._reflect:
            h -= self._spacing
            cr.translate(0, 2 * h + self._reflect.gap)
            cr.scale(1, -1)
            linear = cairo.LinearGradient(0, h, 0, h - self._reflect.height)
            linear.add_color_stop_rgba(0, 0, 0, 0, self._reflect.alpha)
            linear.add_color_stop_rgba(1, 0, 0, 0, 0)
            #h = 2 * h + self._reflect.gap
            h = 0
            for i in range(STARS, 0, -1):
                if i <= self._rating:
                    img = self._current
                else:
                    img = self._nostar
                cr.set_source_pixbuf(img, 0, h)
                cr.mask(linear)
                h += self._size.height + self._spacing

        return True

    def get_width(self):
        return self._size.width

    def get_height(self):
        if self._reflect:
            return (self._size.height * STARS) + (self._spacing * (
                STARS - 1)) + self._reflect.gap + self._reflect.height
        else:
            return (self._size.height * STARS) + (self._spacing * (STARS - 1))

    def check_rating(self, event):
        rating = STARS + 1
        h = 0
        while event.y > h:
            rating -= 1
            h += self._spacing + self._size.height
        return rating

gobject.type_register(HRating)
gobject.type_register(VRating)
