# Copyright (C) 2009-2010 Gloobus Developers
#
#This program is free software; you can redistribute it and/or
#modify it under the terms of the GNU General Public License
#as published by the Free Software Foundation; either version 2
#of the License, or (at your option) any later version.
#
#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

import gtk
import logging
import tarfile, zipfile

from threading import Thread
from xml.dom import minidom
from os import path

from gettext import gettext as _

c = logging.getLogger(__name__)

CANT_READ_FILE_TEXT = "Can't read this file, please select some other file!"
EXTRACT_PATH = path.expanduser("~/.CoverGloobus/themes")
TMP_PATH = "/tmp/"

class ThemeInstallWindow(gtk.Window):

    _imagefile = None
    _themename = None
    _themefile = None
    _themetype = None

    def __init__(self):
        gtk.Window.__init__(self)

        self.set_default_size(300,200)
        self.make_controls()
        self.connect('destroy', lambda w: self.destroy())
        self.show_all()
        self.set_position(gtk.WIN_POS_CENTER)

        self.set_title("Install a Theme")

    def make_controls(self):
        """
            Create buttons and the like
        """
        vbox = gtk.VBox()

        #file select label
        fh = gtk.HBox()
        fl = gtk.Label("Select theme file")
        fh.pack_start(fl,False,False,2)

        #file select button
        fc = gtk.FileChooserButton("Select a file")
        fc.connect("file-set", lambda w: self.on_file_set(w.get_filename()))
        fh.add(fc)

        #theme preview vbox
        tvbox = gtk.VBox()

        #preview thumbnail (gets showed when theme file selected)
        self.thumbnail = gtk.Image()
        tvbox.pack_start(self.thumbnail,False,False,5)
        self.thumbnail.hide()

        #theme no-thumbnail label
        self.ntl = gtk.Label("No screenshot found")
        tvbox.add(self.ntl)

        #theme info label
        self.inl = gtk.Label("No theme information found")
        tvbox.add(self.inl)

        #buttonbox for cancel and ok
        bb = gtk.HBox()
        self.ob = gtk.Button(stock=gtk.STOCK_OK)
        self.ob.connect('clicked', self.on_clicked_ok)
        self.ob.set_size_request(100,25)
        self.ob.set_sensitive(False)
        bb.pack_end(self.ob,False,False,5)

        #cancelbutton
        cb = gtk.Button(stock=gtk.STOCK_CANCEL)
        cb.connect('clicked', lambda w: self.destroy())
        cb.set_size_request(100,25)
        bb.pack_end(cb,False,False,5)

        #pack all in the vbox and add to window
        vbox.pack_start(fh,False,False,0)
        vbox.pack_start(tvbox,True,False,0)
        vbox.pack_start(bb,False,False,0)
        self.add(vbox)


    def set_theme_name(self, f):
        """
            Get theme name from file
            and put in tmp folder
        """
        try:
            if self._themetype == "tar":
                skinfile = f.extractfile(self.get_file_dir(f)+"/skin.xml")
            else:
                skinfile = f.open(self.get_file_dir(f)+"skin.xml")

            dom = minidom.parse(skinfile)
            skinobj = dom.getElementsByTagName("skin")[0]

            self._themename = skinobj.getAttribute("name")
            self.inl.set_label("Theme name: "+self._themename)
        except:
            self._themename = None
            self.inl.set_label(CANT_READ_FILE_TEXT)


    def set_thumbnail(self, f):
        """
            Get thumbnail from the file
            and put in tmp folder
        """
        try:
            c.debug("THEMEINSTALL: Trying to extract thumbnail from file")
            f.extract(self.get_file_dir(f)+"screenshot.png", TMP_PATH)
            self._imagefile = TMP_PATH+self.get_file_dir(f)+"/screenshot.png"
            self.thumbnail.set_from_file(self._imagefile)
            self.thumbnail.show()
            self.ntl.hide()
        except:
            c.error("THEMEINSTALL: Failed extracting thubmnail")
            self.ntl.set_label("Couldn't find a screenshot")
            self._imagefile = None
            self.thumbnail.hide()

    def open_file(self, name):
        """
            Open a specified file and return the object
            Returns the tar object or False
        """
        if tarfile.is_tarfile(name):
            self._themefile = name
            self._themetype = "tar"
            c.debug("THEMINSTALL: Opening a tar file")
            return tarfile.open(name, "r")
        else:
            self._themefile = name
            self._themetype = "zip"
            c.debug("THEMEINSTALL: Opening a zip file")
            return zipfile.ZipFile(name, "r")

    def on_file_set(self, name):
        """
            Gets called when filename is changed
            Execute get_thumbnail and get_information
            and put that in the labels
        """
        try:
            f = self.open_file(name)

            self.set_thumbnail(f)
            self.set_theme_name(f)

            self.ob.set_sensitive(True)
        except:
            c.error("THEMEINSTALL: Can't read this file: "+name)
            self._themefile = None
            self._imagefile = None
            self._themename = None

            self.inl.set_label(CANT_READ_FILE_TEXT)

            self.ntl.hide()
            self.thumbnail.hide()

            self.ob.set_sensitive(False)
            return False

    def on_clicked_ok(self, widget):
        """
            Gets called when ok button is clicked
            Actually installs the theme
        """
        try:
            f = self.open_file(self._themefile)
            f.extractall(EXTRACT_PATH)
            c.debug("THEMEINSTALL: Installed new theme "+self._themename)
            self.destroy()
        except:
            c.error("THEMEINSTALL: Couldn't extract this file!")

    def get_file_dir(self, f):
        """
            Get first directory from file
        """
        if self._themetype == "tar":
            return f.getnames()[0]
        else:
            return f.namelist()[0]
