# Copyright (C) 2009-2010 Gloobus Developers
#
#This program is free software; you can redistribute it and/or
#modify it under the terms of the GNU General Public License
#as published by the Free Software Foundation; either version 2
#of the License, or (at your option) any later version.
#
#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.
#
#You should have received a copy of the GNU General Public License
#along with this program; if not, write to the Free Software
#Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

import cairo
import gobject
import gtk
import logging
import os
import pango

from gettext import gettext as _

from core.common import ABS_PATH
from core.common import ICON_PATH

c = logging.getLogger(__name__)

# ======================= COVERGLOOBUS WINDOW WIDGET ========================= #

class GloobusWindow(gtk.Window):
    def __init__(self):
        gtk.rc_parse(ABS_PATH + '/gloobusgtk.rc')

        # initilizing propertries
        self._to_resize_id = None
        self._resize_show = False
        self._roundness = 10
        self._opacity = 0.8
        self._gap_top = self._roundness/2 + 16
        self._gap_bottom = self._roundness/2
        self._gap_right = self._roundness/2
        self._gap_left = self._roundness/2

        # initializing window
        gtk.Window.__init__(self)
        self._new_background()
        self.set_decorated(False)
        self.set_app_paintable(True)
        self.set_colormap( self.get_screen().get_rgba_colormap())
        self.set_border_width(0)
        #self.set_icon_from_file(ICON_PATH)
        self.set_icon_name('covergloobus')
        self._init_buttons()

        self.set_size_request(50, 50)

        self.add_events(gtk.gdk.BUTTON_PRESS_MASK)
        self.connect('expose_event',       self.expose_event)
        self.connect('configure_event',    self.configure_event)
        self.connect('button_press_event', self.button_press_event)

        width, height = self.get_size()
        self.container = gtk.VBox()
        self.container.set_size_request(width-self._gap_left-self._gap_right,
            height-self._gap_top-self._gap_bottom)

        self.fixed = gtk.Fixed()
        self.fixed.put(self.btn_close,0,0)
        self.fixed.put(self.btn_resize,width,height)
        self.fixed.put(self.container,self._gap_left,self._gap_top)
        self._place_buttons()
        gtk.Window.add(self, self.fixed)

    def _new_background(self, width=None, height=None):
        if not width or not height:
            width, height = self.get_size()
        self._background_width = width
        self._background_height = height
        self._background = cairo.ImageSurface(cairo.FORMAT_ARGB32,
            self._background_width, self._background_height)
        self.draw_background(cairo.Context(self._background),
            self._background_width, self._background_height)

    def _init_buttons(self):
        self.btn_close = None
        self.btn_menu = None
        self.btn_resize = None
        self.menu = None

        close_img = gtk.Image()
        close_img.set_from_file(ABS_PATH + '/images/close_button.png')
        self.btn_close = gtk.EventBox()
        self.btn_close.set_visible_window(False)
        self.btn_close.add(close_img)
        self.btn_close.set_size_request(16, 16)
        self.btn_close.connect("button-press-event", lambda w,e: self.destroy())

        resize_img = gtk.Image()
        resize_img.set_from_file(ABS_PATH + '/images/resize_button.png')
        self.btn_resize = gtk.EventBox()
        self.btn_resize.set_visible_window(False)
        self.btn_resize.add(resize_img)
        self.btn_resize.set_size_request(16, 16)
        self.btn_resize.connect("button-press-event",
            lambda w,e: self.begin_resize_drag(gtk.gdk.WINDOW_EDGE_SOUTH_EAST,
                e.button, int(e.x_root), int(e.y_root), e.get_time()))

    def _place_buttons(self):
        width, height = self.get_size()
        self.fixed.move(self.btn_close, width-(self._gap_left+16),
            self._gap_top-16)
        if self._resize_show:
         self.fixed.move(self.btn_resize, width-(self._gap_left+16),
            height-self._gap_bottom)

    # =============================== EVENTS ================================ #

    def expose_event(self, widget, event):
        cr = self.window.cairo_create()
        cr.rectangle(event.area.x, event.area.y,
            event.area.width, event.area.height)
        cr.clip()
        cr.set_operator(cairo.OPERATOR_CLEAR)
        cr.paint()
        cr.set_operator(cairo.OPERATOR_OVER)
        cr.set_source_surface(self._background)
        cr.paint()
        return False

    def configure_event(self, widget, event):
        if self._background_width != event.width or self._background_height != event.height:
            self._new_background(event.width, event.height)
            self._place_buttons()
            self.container.set_size_request(event.width-self._gap_left-self._gap_right,
                event.height-self._gap_top-self._gap_bottom)
            self.queue_draw()
        return False

    def button_press_event(self,widget,event):
        if event.button == 1:
            self.begin_move_drag(event.button, int(event.x_root), int(event.y_root), event.time)
        #elif event.button == 2:
            #pass
        #elif event.button == 3:
            #if self.menu:
                #self.menu.popup(None,None,None,event.button,event.get_time())

    def draw_background(self, cr, w, h):
        r = self._roundness
        x = 0
        y = 0

        cr.set_operator(cairo.OPERATOR_CLEAR)
        cr.move_to    ( x+r,y )                       # Move to A
        cr.line_to    ( x+w-r,y )                        # Straight line to B
        cr.curve_to   ( x+w-r,y,x+w,y,x+w,y+r )        # Curve to C
        cr.line_to    ( x+w,y+h-r )                    # Move to D
        cr.curve_to   ( x+w,y+h-r,x+w,y+h,x+w-r,y+h )    # Curve to E
        cr.line_to    ( x+r,y+h )                      # Line to F
        cr.curve_to   ( x+r,y+h,x,y+h,x,y+h-r )       # Curve to G
        cr.line_to    ( x,y+r )                          # Line to H
        cr.curve_to   ( x,y+r,x,y,x+r,y)              # Curve to G
        cr.fill_preserve()

        cr.set_operator(cairo.OPERATOR_OVER)
        cr.set_source_rgba (0, 0, 0, self._opacity)
        cr.fill()

    # ============================ PUBLIC METHODS =========================== #

    def add(self, widget):
        self.container.add(widget)

    def menu_add(self, item):
        if not self.menu:
            self.menu   = gtk.Menu()
            menu_img = gtk.Image()
            menu_img.set_from_file(ABS_PATH + '/images/menu_button.png')
            self.btn_menu = gtk.EventBox()
            self.btn_menu.set_visible_window(False)
            self.btn_menu.add(menu_img)
            self.btn_menu.set_size_request(16, 16)
            self.btn_menu.connect_object("event", self.btn_menu_press, self.menu)
            self.fixed.put(self.btn_menu, 4, 4)
        self.menu.append(item)
        item.show()

    def btn_menu_press(self, widget, event):
        if event.type == gtk.gdk.BUTTON_PRESS:
            self.menu.popup(None,None,None,event.button,event.get_time())
            return True
        else:
            return False

    def resize_show(self, show):
        if self._resize_show != show:
            if show:
                self._gap_bottom = self._gap_bottom + 16
                width, height = self.get_size()
                if not self.btn_resize:

                    self.fixed.put(self.btn_resize, width-20, height-self._gap_bottom)
                else:
                    self.btn_resize.show()
                    self.fixed.move(self.btn_resize, width-20, height-self._gap_bottom)
            else:
                self._gap_bottom = self._gap_bottom - 16
                self.btn_resize.hide()
            self._resize_show = show

    def show_all(self):
        gtk.Window.show_all(self)
        if not self._resize_show:
            self.btn_resize.hide()

    def set_size_request(self, width, height):
        gtk.Window.set_size_request(self,
            width + self._gap_left + self._gap_right,
            height + self._gap_top + self._gap_bottom)

    def resize(self, width, height):
        gtk.Window.resize(self,
            width + self._gap_left + self._gap_right,
            height + self._gap_top + self._gap_bottom)

    def smooth_resize(self, w, h):
        print 'resize', w, 'x', h
        width, height = self.get_size()
        print 'resize from', width, 'x', height
        if w > width:
            self._to_resize_way_w = 1
        else:
            self._to_resize_way_w = -1
        self._to_resize_width = self._to_resize_way_w*(w - width)
        print self._to_resize_width
        if h > height:
            self._to_resize_way_h = 1
        else:
            self._to_resize_way_h = -1
        self._to_resize_height = self._to_resize_way_h*(h - height)
        print self._to_resize_height
        self._to_resize_step_w = self._to_resize_width/10
        print self._to_resize_step_w
        self._to_resize_step_h = self._to_resize_height/10
        print self._to_resize_step_h
        if self._to_resize_id:
            gobject.source_remove(self._to_resize_id)
        self._to_resize_id = gobject.timeout_add(25, self._smooth_resize)

    def _smooth_resize(self):
        width, height = self.get_size()
        if self._to_resize_width <= self._to_resize_step_w or self._to_resize_height <= self._to_resize_step_h:
            self.resize(width + self._to_resize_way_w*self._to_resize_width,
                height + self._to_resize_way_h*self._to_resize_height)
            self._to_resize_id = None
            return False
        else:
            self.resize(width + self._to_resize_way_w*self._to_resize_step_w,
                height + self._to_resize_way_h*self._to_resize_step_h)
            self._to_resize_width = self._to_resize_width - self._to_resize_step_w
            self._to_resize_height = self._to_resize_height - self._to_resize_step_h
            return True

# ======================= COVERGLOOBUS COVER WINDOW ========================== #

class CoverWindow(GloobusWindow):
    def __init__(self, track):
        GloobusWindow.__init__(self)
        if not os.path.exists(track.cover):
            common.c.warning('ShowCover: cover file not found')
            self.destroy()
            return

        self.set_title(track.album)
        self.set_icon_from_file(ICON_PATH)

        self.connect('button_press_event', self.clicked)
        self.add_events (gtk.gdk.KEY_PRESS_MASK )
        self.connect('key-press-event', self.keypressed)
        self.set_position(gtk.WIN_POS_CENTER)

        self.cover = gtk.gdk.pixbuf_new_from_file(track.cover)
        self.calc_cover_size()
        w = self.cover.get_width()
        h = self.cover.get_height()
        self.set_size_request(w,h)

        image = gtk.Image()
        image.set_from_pixbuf(self.cover)

        self.add(image)
        self.show_all()

    def calc_cover_size(self):
        width = gtk.gdk.screen_width()-110
        height = gtk.gdk.screen_height()-130

        ratio = 0
        w = self.cover.get_width()
        h = self.cover.get_height()
        if w > width:
            ratio = float(w) / float(width)
        if h > height:
            r = float(h) / float(height)
            if r > ratio:
                ratio = r
        if ratio:
            self.cover = self.cover.scale_simple(int(w/ratio), int(h/ratio), gtk.gdk.INTERP_BILINEAR)

    def clicked(self,widget,event):
        if event.button == 1:
            if event.type == gtk.gdk._2BUTTON_PRESS:
                self.destroy()

    def keypressed(self, widget, event):
        if event.keyval == gtk.keysyms.Escape:
            self.destroy()

# =============================== LYRIC WINDOW =============================== #

class LyricWindow(GloobusWindow):
    def __init__(self, text='CoverGloobus', url='http://gloobus.net/'):
        GloobusWindow.__init__(self)
        self.text = LyricText(text)
        self.url = url
        self.calc_sizes()

        self.scroll = gtk.ScrolledWindow()
        viewport = gtk.Viewport()
        viewport.add(self.text)
        viewport.set_border_width(0)
        viewport.set_shadow_type(gtk.SHADOW_NONE)
        self.scroll.add(viewport)
        self.scroll.set_policy(gtk.POLICY_AUTOMATIC, gtk.POLICY_AUTOMATIC)
        self.scroll.set_shadow_type(gtk.SHADOW_NONE)
        self.scroll.set_size_request(self.width,self.height-25)

        self.button = LyricLabel(_("Go to webpage"))
        self.web = gtk.EventBox()
        self.web.set_visible_window(False)
        self.web.add(self.button)
        self.web.connect('button-press-event', lambda w,e: os.system("gnome-open " + self.url))
        self.web.connect("enter-notify-event", lambda w,e: self.window.set_cursor(gtk.gdk.Cursor(gtk.gdk.HAND2)))
        self.web.connect("leave-notify-event", lambda w,e: self.window.set_cursor(None))

        self.create_menu()

        #self.add(self.scroll)
        #self.add(self.web)
        self.container.pack_start(self.scroll)
        self.container.pack_end(self.web, False)

        self._widget = False
        self.loader = None
        self._ontop = False
        self.connect('button_press_event', self.clicked)

    def create_menu(self):

        toclipboard = gtk.MenuItem(_("Copy to clipboard"))
        toclipboard.connect('activate', lambda w: gtk.clipboard_get().set_text(self.text.get_text()))
        self.menu_add(toclipboard)

        save = gtk.MenuItem(_("Save to file"))
        save.connect('activate', self.save_to_file)
        self.menu_add(save)

        self.menu_add(gtk.SeparatorMenuItem())

        ontop = gtk.CheckMenuItem(_("Always on top"))
        ontop.connect('activate', self.set_ontop)
        self.menu_add(ontop)

        self.menu_add(gtk.SeparatorMenuItem())

        close = gtk.MenuItem(_("Close"))
        close.connect('activate', lambda w: self.destroy())
        self.menu_add(close)


    def set_widget(self, widget):
        if widget != self._widget:
            self._widget = widget
            if widget:
                self.stick()
                self.set_property("skip-taskbar-hint", True)
                self.set_property("skip-pager-hint", True)
                self.set_keep_below(True)
            else:
                self.unstick()
                self.set_property("skip-taskbar-hint", False)
                self.set_property("skip-pager-hint", False)
                self.set_keep_below(False)

    def set_loader(self, loader):
        if self.loader:
            self.loader.stop()
        self.loader = loader

    def set_waiting(self, wait):
        self.set_text(wait, 'http://gloobus.net/')

    def set_text(self, text, url):
        self.text.set_text(text)
        self.calc_sizes()
        #self.scroll.set_size_request(self.width,self.height-25)
        self.url = url

    def calc_sizes(self):
        tw = self.text.w+20
        th = self.text.h+45
        sw = gtk.gdk.screen_width()-200
        sh = gtk.gdk.screen_height()-200
        if tw < sw:
            self.width = tw
        else:
            self.width = sw

        if th < sh:
            self.height = th
        else:
            self.height = sh

        #self.smooth_resize(self.width,self.height)
        self.resize(self.width,self.height)

    def save_to_file(self, widget):
        text = self.text.get_text()
        chooser = gtk.FileChooserDialog('CoverGloobus - Save to ...',self,
            gtk.FILE_CHOOSER_ACTION_SAVE,
            (gtk.STOCK_CANCEL,gtk.RESPONSE_CANCEL,gtk.STOCK_SAVE,gtk.RESPONSE_OK))
        chooser.set_icon_from_file(ICON_PATH)
        response = chooser.run()
        if response == gtk.RESPONSE_OK:
            filename = chooser.get_filename()
        else:
            filename = None
        chooser.destroy()
        if filename and os.path.exists(filename):
            md = gtk.Dialog('', self, gtk.DIALOG_DESTROY_WITH_PARENT,
                (gtk.STOCK_CANCEL,gtk.RESPONSE_CANCEL,gtk.STOCK_OK,gtk.RESPONSE_OK))
            label = gtk.Label(_('A file named "{0}" already exists.  Do you want to replace it?').format(os.path.basename(filename)))
            md.vbox.pack_start(label)
            label.show()
            response = md.run()
            md.destroy()
            if response == gtk.RESPONSE_CANCEL:
                filename = None
        if filename:
            f = open(filename, 'w')
            f.write(text)
            f.close()

    def clicked(self, widget, event):
        if self._widget and not self._ontop:
            self.set_keep_below(True)

    def set_ontop(self, widget):
        self._ontop = widget.get_active()
        self.set_keep_above(self._ontop)
        if not self._ontop:
            self.set_keep_below(True)

# =============================== LYRIC TEXT ================================= #

class LyricText(gtk.DrawingArea):
    def __init__(self, text):
        gtk.DrawingArea.__init__(self)
        self.connect("expose_event", self.expose)
        self.set_text(text)

    def set_text(self, text):
        self.text = text.strip().decode('utf-8')
        self.calc_size()
        self.queue_draw()

    def get_text(self):
        return self.text

    def expose(self, widget, event):
        cr = widget.window.cairo_create()
        cr.set_operator(cairo.OPERATOR_CLEAR)
        cr.paint()
        cr.set_operator(cairo.OPERATOR_OVER)
        cr.set_source_rgba (0, 0, 0, 0.8)
        cr.paint()
        self.draw(cr)
        return False

    def draw(self, cr):
        #cr.select_font_face("Lucida Grande", cairo.FONT_SLANT_NORMAL, cairo.FONT_WEIGHT_BOLD)
        cr.select_font_face("Liberation Mono", cairo.FONT_SLANT_NORMAL, cairo.FONT_WEIGHT_BOLD)
        #cr.set_font_size(12)
        cr.set_font_size(14)
        x = 5
        y = 15
        for line in self.text.splitlines():
            # first some text shadow
            cr.move_to(x-1,y)
            cr.set_source_rgba(0.0, 0.0, 0.0,0.9)
            cr.show_text(line)
            # now white text
            cr.move_to(x,y)
            cr.set_source_rgb(1.0, 1.0, 1.0)
            cr.show_text(line)
            y = y + 15

    def calc_size(self):
        lines = len(self.text.splitlines())
        self.h = lines*15+12
        width = 20
        for line in self.text.splitlines():
            size = len(line)
            if size > width:
                width = size
        self.w = width*8+10
        self.set_size_request(self.w, self.h)

# ============================== LYRIC LABEL ================================= #

class LyricLabel(gtk.Label):
    def __init__(self, label):
        gtk.Label.__init__(self)
        self.modify_font(pango.FontDescription("Liberation Mono 12 bold"))
        self.set_markup('<span foreground="white">'+label+'</span>')
